#include <stdint.h>
#include "built_in.h"
#include "fonts.h"

// TFT module connections
sbit TFT_BLED at GPIOA_PDOR.B4;
sbit TFT_CS at GPIOB_PDOR.B20;
char TFT_DataPort at GPIOD_PDOR;
sbit TFT_RD at GPIOC_PDOR.B8;
sbit TFT_RS at GPIOC_PDOR.B2;
sbit TFT_RST at GPIOB_PDOR.B19;
sbit TFT_WR at GPIOC_PDOR.B9;
// End TFT module connections

unsigned char adc_conv_res[12];
uint16_t ADC_Result = 0;


void Write_to_Data_Lines(unsigned char _hi, unsigned char _lo) {
  GPIOD_PDOR = (unsigned int) (_lo | (_hi << 8));
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Write_to_Data_Lines(0, index);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Write_to_Data_Lines(0, cmd);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Write_to_Data_Lines(Hi(_data), Lo(_data));
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}


static void InitDisplay() {
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init_SSD1963_Board_50(800, 480);
//  TFT_Rotate_180(180);
  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Font(&ISOCPEUR26x46,CL_BLACK,FO_HORIZONTAL);
  TFT_Write_Text("MM5 KINETIS ADC TEST",200,25);
  TFT_Write_Text("----------------------",200,40);

}
void main() {

   InitDisplay();
   ADC1_Init();                                                // Init ADC1
   ADC1_CFG2.MUXSEL = 1;
   ADC1_Set_Input_Channel(_ADC_CHANNEL_10);                    // Choose ADC channel 12 for PTB6
   Delay_ms(100);
   
     while(1) {
       ADC_Result = ADC1_Get_Sample(10);
       WordToStr(ADC_Result,adc_conv_res);
       TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
       TFT_Set_Pen(CL_WHITE,1);
       TFT_Set_Font(&ISOCPEUR26x46,CL_BLACK,FO_HORIZONTAL);
       TFT_Rectangle(85,100,200,190);
       TFT_Write_Text(adc_conv_res, 85, 100);
       Delay_ms(10);
   }
}