#include <stdint.h>
#include "built_in.h"


// TFT module connections
sbit TFT_BLED at GPIOA_PDOR.B4;
sbit TFT_CS at GPIOB_PDOR.B20;
char TFT_DataPort at GPIOD_PDOR;
sbit TFT_RD at GPIOC_PDOR.B8;
sbit TFT_RS at GPIOC_PDOR.B2;
sbit TFT_RST at GPIOB_PDOR.B19;
sbit TFT_WR at GPIOC_PDOR.B9;
// End TFT module connections

#define ARRAY_LENGHT 50

char someData[ARRAY_LENGHT];
char data_[10];
char rData[ARRAY_LENGHT];
char i;


void EEPROM_24C02_WrSingle(unsigned short wAddr, unsigned short wData) {
  data_[0] = wAddr;
  data_[1] = wData;
  I2C_Start();
  // issue I2C start signal
  I2C_Write(0x50,data_,2,END_MODE_STOP);
}//~

//--------------- Reads data from 24C02 EEPROM - single location (random)
unsigned short EEPROM_24C02_RdSingle(unsigned short rAddr) {
  data_[0] = rAddr;
  I2C_Start();              // issue I2C start signal
  I2C_Write(0x50,data_,1,END_MODE_RESTART);
  I2C_Read(0x50,data_,1,END_MODE_STOP);

  return data_[0];
}


void Write_to_Data_Lines(unsigned char _hi, unsigned char _lo) {
  GPIOD_PDOR = (unsigned int) (_lo | (_hi << 8));
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Write_to_Data_Lines(0, index);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Write_to_Data_Lines(0, cmd);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Write_to_Data_Lines(Hi(_data), Lo(_data));
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}




static void InitDisplay() {
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init_SSD1963_Board_50(800, 480);
//  TFT_Rotate_180(180);

  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Write_Text("MM5 KINETIS I2C TEST",100,25);
  TFT_Write_Text("--------------------",100,40);

}

void SystemInit()
{
  InitDisplay();
  GPIO_Digital_Output(&GPIOE_PDOR, _GPIO_PINMASK_8);                    // Set PB8 as digital output
  Sound_Init(&GPIOE_PDOR, 8);
}
int j,k = 50;
void main() {
  short i = 0;
  short flag = 0;
  unsigned short testArray[10] = {0};
  char string[10] = {0};
  SystemInit();
  I2C2_Init_Advanced(100000, &_GPIO_Module_I2C2_PA12_13);        // performs I2C initialization

  memset(someData, 0x00, sizeof(someData));
  memset(rData, 0x01, sizeof(someData));               // something not zero

  Delay_ms(100);
  TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
  TFT_Set_Pen(CL_WHITE,1);
  TFT_Rectangle(85,100,200,190);
  TFT_Write_Text("Erasing", 55, 60);

  for(i = 0; i < ARRAY_LENGHT; i++) {
    EEPROM_24C02_WrSingle(i,0x00);
    Delay_ms(5);
  }


  for(i = 0x00; i < sizeof(someData); i++){
    rData[i] = EEPROM_24C02_RdSingle(i);
    Delay_ms(10);
  }

  if (!memcmp(someData , rData, sizeof(someData))){
      Sound_Play(1000, 100);
    Delay_ms(100);
  }

  TFT_Write_Text("Writting", 55, 80);

  for(i = 0; i < sizeof(someData); i++) {
    EEPROM_24C02_WrSingle(i,i);
    someData[i] = i;
    Delay_ms(5);
  }

  TFT_Write_Text("Complete!", 120, 80);
  TFT_Write_Text("Reading", 55, 100);

  for(i = 0x00; i < sizeof(someData); i++){
    rData[i] = EEPROM_24C02_RdSingle(i);
    Delay_ms(10);
  }

  if (!memcmp(someData , rData, sizeof(someData))){
    Sound_Play(1000, 100);
    Delay_ms(300);
    Sound_Play(1000, 100);
    TFT_Write_Text("Test complete!",55,120);
  }
  else {
    Sound_Play(300, 300);
  }

  while(1)
    ;
}