#include <stdint.h>
#include "built_in.h"

// TFT module connections
sbit TFT_BLED at GPIOA_PDOR.B4;
sbit TFT_CS at GPIOB_PDOR.B20;
char TFT_DataPort at GPIOD_PDOR;
sbit TFT_RD at GPIOC_PDOR.B8;
sbit TFT_RS at GPIOC_PDOR.B2;
sbit TFT_RST at GPIOB_PDOR.B19;
sbit TFT_WR at GPIOC_PDOR.B9;
// End TFT module connections

const short _C3DHALL7_REG_DEVICE_ID = 0;

sbit SPI_CS at GPIOA_PDOR.B5;


void Write_to_Data_Lines(unsigned char _hi, unsigned char _lo) {
  GPIOD_PDOR = (unsigned int) (_lo | (_hi << 8));
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Write_to_Data_Lines(0, index);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Write_to_Data_Lines(0, cmd);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Write_to_Data_Lines(Hi(_data), Lo(_data));
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}




static void InitDisplay() {
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init_SSD1963_Board_50(800, 480);

  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Write_Text("MM5 KINETIS SPI TEST",100,25);
  TFT_Write_Text("--------------------",100,40);

}


typedef struct
{
    uint8_t company_id;
    uint8_t device_id;

}T_C3DHALL7_DEV_INFO;

T_C3DHALL7_DEV_INFO info;


static void hal_spiWrite(uint8_t *pBuf, uint16_t nBytes)
{
    uint8_t *ptr = pBuf;
    while( nBytes-- )
        SPI_Write( *( ptr++ ) );
}

static void hal_spiRead(uint8_t *pBuf, uint16_t nBytes)
{
    uint8_t *ptr = pBuf;
    while( nBytes-- )
        *( ptr++ ) = SPI_Read( 0x00 );
}

void c3dhall7_readData(uint8_t reg, uint8_t *dataOut, uint8_t nBytes){
  uint8_t writeReg[ 1 ];

  writeReg[ 0 ] = reg | 0x80;

  SPI_CS = 0;
  hal_spiWrite( writeReg, 1 );
  hal_spiRead( dataOut, nBytes );
  SPI_CS = 1;

}

void c3dhall7_deviceInfo(T_C3DHALL7_DEV_INFO *info)
{
    uint8_t readData[ 2 ];

    c3dhall7_readData(_C3DHALL7_REG_DEVICE_ID, readData, 2);

    info->company_id = readData[ 0 ];
    info->device_id = readData[ 1 ];
}

void SystemInit()
{
    GPIO_Digital_Output(&GPIOA_PDOR, _GPIO_PINMASK_5);
    GPIO_Digital_Output(&GPIOE_PDOR, _GPIO_PINMASK_8);                    // Set PB8 as digital output
    Sound_Init(&GPIOE_PDOR, 8);

  
  SPI_CS = 1;
  Delay_100ms();

  SPI2_Init_Advanced(8000000, _SPI_CFG_MASTER |
                              _SPI_CFG_FRAME_SIZE_8BITS |
                              _SPI_CFG_CLK_IDLE_LOW |
                              _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                              _SPI_CFG_MSB_FIRST, &_GPIO_Module_SPI2_PB21_23_22);
  Delay_100ms();
  InitDisplay();
}

void main() {
  short i = 0;
  short flag = 0;
  unsigned short testArray[10] = {0};
  char string[10] = {0};
  
  
  SystemInit();
  
  for(i = 0; i < 10; i++)
  {
    c3dhall7_deviceInfo(&info);
    testArray[i] = info.device_id;
    Delay_ms(200);
  }
  for(i = 0; i < 10; i++)
  {
    if(testArray[i] != 192)
      flag = 1;
  }
  if(flag)  {
  TFT_Write_Text("TEST FAILED!",100,100);
  Sound_Play(300, 300);
  }
  else{
  TFT_Write_Text("TEST OK!",100,100);
  Sound_Play(1000, 100);
  Delay_ms(300);
  Sound_Play(1000, 100);
  }

  while(1)
    ;
}