/*
    mmb_---.h

-----------------------------------------------------------------------------

  This file is part of mikroSDK.
  
  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

/**
        \file   mmb_---.h

- Version               :   1.0.0
- Date                  :   May 2019
- Developer             :   Milos Vidojevic

    Generated from mmb---.h template.

    ---

    Template for definition of MCU connections used on mikromedia systems. This 
    file should be used as start point for board definition file development. 

    Hints :

    - Copy file inside [project base directory]/portable.
    - Replace all "---" appearances with proper values.

*/
/* -------------------------------------------------------------------------- */

#include "compiler.h"

#ifndef _MMB_5_KINETIS_
#define _MMB_5_KINETIS_

/*
    REF SEL jumper position :

    3V3         - 1
    2.048V      - 0
*/

#define JP_VREF                     0

/*
    MP3 jumper position :

    LIN         - 1
    MIC         - 0
*/

#define JP_MP3                      1

/*
    ACCEL jumpers position.
*/

#define JP_ACCEL_A                  0
#define JP_ACCEL_B                  0

/*
    TFT jumpers position.
*/

#define JP_TFT_LR                   1
#define JP_TFT_UD                   0
#define JP_TFT_DITH                 1
#define JP_TFT_MODE                 0

// ============================================================================
//                                                                    DISPLAY
// ============================================================================

//extern sfr sbit TFT_BLED;
extern sfr sbit TFT_CS;
extern sfr sbit TFT_RD;
extern sfr sbit TFT_RS;
extern sfr sbit TFT_RST;
extern sfr sbit TFT_WR;

static void Write_to_Data_Lines( unsigned char _hi, unsigned char _lo )
{
    GPIOD_PDOR = (unsigned int) (_lo | (_hi << 8));
}

static void Set_Index( unsigned short index )
{
    TFT_RS = 0;
    Write_to_Data_Lines(0, index);
    TFT_WR = 0;
    asm nop;
    TFT_WR = 1;
}

static void Write_Command( unsigned short cmd )
{
    TFT_RS = 1;
    Write_to_Data_Lines(0, cmd);
    TFT_WR = 0;
    asm nop;
    TFT_WR = 1;
}

static void Write_Data( unsigned int _data )
{
    TFT_RS = 1;
//    Write_to_Data_Lines(Hi(_data), Lo(_data));
    Write_to_Data_Lines( ( _data >> 8 ), ( _data & 0x00FF ) );
    TFT_WR = 0;
    asm nop;
    TFT_WR = 1;
}

static void hal_display_init ( void )
{
    //  Initialize display.

    TFT_Set_Default_Mode( );
    TFT_Set_Active( Set_Index, Write_Command, Write_Data );
    TFT_Init_SSD1963_Board_50( 800, 480 );
    TFT_Set_DBC_SSD1963( 255 );
}

// ============================================================================
//                                                                      TOUCH
// ============================================================================

static uint8_t hal_touch_init ( void )
{
//  Initialize touch senor.
    GPIO_Digital_Output(&GPIOB_PDOR, _GPIO_PINMASK_3);
    GPIO_Digital_Input (&GPIOB_PDIR, _GPIO_PINMASK_2);
    GPIOB_PDOR.B3 = 1;
    while (GPIOB_PDIR.B2 == 0)
    {
      GPIOB_PDOR.B3 = 0;
      Delay_us(10);
      GPIOB_PDOR.B3 = 1;
      Delay_us(10);
    }

    I2C0_Init_Advanced( 100000, &_GPIO_Module_I2C0_PB2_3 );

    FT5XX6_SetI2CAddress(0x38);
    
    if (FT5XX6_IsOperational( ) != FT5XX6_OK)
    {
        return  FT5XX6_FAILURE;
    }
    
    FT5XX6_SetDefaultMode( );
    FT5XX6_SetController( _TC_FT5X06 );
    FT5XX6_SetSizeAndRotation( 800,480,2 );
  
    return FT5XX6_OK;
}

// ============================================================================
//                                                                    CONSOLE
// ============================================================================

//  Map proper functions used with console.

//#define CONSOLE_WRITE_TXT             UART3_Write_Text
//#define CONSOLE_READY                 UART3_Data_Ready
//#define CONSOLE_READ                  UART3_Read

#define CONSOLE_WRITE_TXT             UART4_Write_Text
#define CONSOLE_READY                 UART4_Data_Ready
#define CONSOLE_READ                  UART4_Read

static void hal_console_init ( void )
{
    //  Initialize console.

    /*UART3_Init_Advanced(
                         115200,
                         _UART_8_BIT_DATA,
                         _UART_NOPARITY,
                         _UART_ONE_STOPBIT,
                         &_GPIO_Module_UART3_PE4_5
                        );*/
                        
    UART4_Init_Advanced(
                         115200,
                         _UART_8_BIT_DATA,
                         _UART_NOPARITY,
                         _UART_ONE_STOPBIT,
                         &_GPIO_Module_UART4_PC14_15
                        );
}

// ============================================================================
//                                                                      POWER
// ============================================================================

#if ( _MMB_MODULE_POWER == 1 )

//  Detects external power source on TB1 / CN9.

static sbit BSP_DC_STATUS at                GPIOE_PDIR.B28;

//  Detects charging of the battery.

static sbit BSP_CHARGER_STATUS at           GPIOE_PDIR.B27;

#if ( JP_VREF == 0 )
#   define BSP_POWER_ADC_VREF               ( 2.048f )
#elif ( JP_VREF == 1 )
#   define BSP_POWER_ADC_VREF               ( 3.300f )
#else
#   error "REF SEL POSITION UNKNOWN"
#endif

#define BSP_POWER_ADC_CHANNEL               ( 13 )
#define BSP_POWER_ADC_RESOLUTION            ( 4096.0f )

const static hal_adc_obj_t BSP_POWER_ADC_MODULE =
{
    ADC1_Read,
    ADC1_Get_Sample
};

static void hal_power_init ( void )
{
    //  Initialize BSP_DC_STATUS as input pin

    GPIO_Digital_Input( &GPIOE_PDIR, _GPIO_PINMASK_28 );

    //  Initialize BSP_CHARGER_STATUS as input pin

    GPIO_Digital_Input( &GPIOE_PDIR, _GPIO_PINMASK_27 );

    //  Initialize ADC module

    MPU_CESR = 0;
    SIM_SCGC3 = 0x8000000;
    ADC1_SC1A = 0x10;
    ADC1_CFG1.MODE = 0x10;
    Delay_100ms( );

    ADC1_Set_Input_Channel( BSP_POWER_ADC_CHANNEL );

    ADC1_Init( );
}

#endif

// ============================================================================
//                                                                    MICROSD
// ============================================================================

#if ( _MMB_MODULE_MICROSD == 1 )

static void hal_microsd_init ( void )
{
    Mmc_Set_Interface(_MMC_INTERFACE_SDHC);
}
#endif

// ============================================================================
//                                                                      FLASH
// ============================================================================

#if ( _MMB_MODULE_FLASH == 1 )

static sbit BSP_FLASH_CE at                 GPIOC_PDOR.B19;

const static hal_spi_obj_t BSP_FLASH_SPI_MODULE =
{
    SPI1_Write,
    SPI1_Read
};

static void hal_flash_init ( void )
{
    //  Initialize BSP_FLASH_CE as output pin.

    GPIO_Digital_Output( &GPIOC_PDOR, _GPIO_PINMASK_19 );

    //  Initialize SPI module.

    SPI1_Init_Advanced( 1000000,

                        _SPI_CFG_MASTER |
                        _SPI_CFG_FRAME_SIZE_8BITS |
                        _SPI_CFG_CLK_IDLE_LOW |
                        _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                        _SPI_CFG_MSB_FIRST,

                        &_GPIO_Module_SPI1_PB11_17_16 );
                        
    SPI_Set_Active( SPI1_Read, SPI1_Write );

    BSP_FLASH_CE = 1;
}

#endif

// ============================================================================
//                                                                         RF
// ============================================================================

#if ( _MMB_MODULE_RF == 1 )

static sbit BSP_RF_CS  at                           GPIOA_PDOR.B26;
static sbit BSP_RF_CE  at                           GPIOA_PDOR.B25;
static sbit BSP_RF_INT at                           GPIOA_PDIR.B27;

const static hal_spi_obj_t BSP_RF_SPI_MODULE =
{
    SPI0_Write,
    SPI0_Read
};

static void hal_rf_init ( void )
{
    //  Initialize BSP_RF_INT as input pin.

    GPIO_Digital_Input( &GPIOA_PDIR, _GPIO_PINMASK_27 );

    //  Initialize BSP_RF_CE as output pin.

    GPIO_Digital_Output( &GPIOA_PDOR, _GPIO_PINMASK_25 );

    //  Initialize BSP_RF_CS as output pin.

    GPIO_Digital_Output( &GPIOA_PDOR, _GPIO_PINMASK_26 );

    //  Initialize SPI module.

    SPI0_Init_Advanced( 1000000,

                        _SPI_CFG_MASTER |
                        _SPI_CFG_FRAME_SIZE_8BITS |
                        _SPI_CFG_CLK_IDLE_LOW |
                        _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                        _SPI_CFG_MSB_FIRST,

                        &_GPIO_Module_SPI0_PC5_7_6 );

    SPI_Set_Active( SPI1_Read, SPI1_Write );
}
#endif

// ============================================================================
//                                                                        MP3
// ============================================================================

#if ( _MMB_MODULE_MP3 == 1 )

static volatile sbit BSP_MP3_DREQ at                 GPIOA_PDIR.B11;
static volatile sbit BSP_MP3_CS   at                 GPIOA_PDOR.B6;
static volatile sbit BSP_MP3_RST  at                 GPIOA_PDOR.B10;
static volatile sbit BSP_MP3_DCS  at                 GPIOA_PDOR.B24;

static const hal_spi_obj_t BSP_MP3_SPI_MODULE =
{
    SPI0_Write,
    SPI0_Read
};

static void hal_mp3_init ( void )
{
    //  Initialize BSP_MP3_DREQ as input pin.

    GPIO_Digital_Input( &GPIOA_PDIR, _GPIO_PINMASK_11 );

    //  Initialize BSP_MP3_CS as output pin.

    GPIO_Digital_Output( &GPIOA_PDOR, _GPIO_PINMASK_6 );

    //  Initialize BSP_MP3_RST as output pin.

    GPIO_Digital_Output( &GPIOA_PDOR, _GPIO_PINMASK_10 );

    //  Initialize BSP_MP3_DCS as output pin.

    GPIO_Digital_Output( &GPIOA_PDOR, _GPIO_PINMASK_24 );

    //  Initialize SPI module.

    SPI0_Init_Advanced( 10000000,

                        _SPI_CFG_MASTER |
                        _SPI_CFG_FRAME_SIZE_8BITS |
                        _SPI_CFG_CLK_IDLE_LOW |
                        _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                        _SPI_CFG_MSB_FIRST,

                        &_GPIO_Module_SPI0_PC5_7_6 );
                        
    SPI_Set_Active( SPI1_Read, SPI1_Write );
}

#endif

// ============================================================================
//                                                                      ACCEL
// ============================================================================

#if ( _MMB_MODULE_ACCEL == 1 )

static sbit BSP_ACCEL_INT_1 at                          GPIOA_PDOR.B16;
static sbit BSP_ACCEL_INT_2 at                          GPIOA_PDOR.B17;

const static hal_i2c_obj_t BSP_ACCEL_I2C_MODULE =
{
    I2C0_Start,
    I2C0_Write,
    I2C0_Read
};

static void hal_accel_init ( void )
{
    //  Initialize BSP_ACCEL_INT_1 as input pin.

    GPIO_Digital_Input( &GPIOA_PDOR, _GPIO_PINMASK_16 );

    //  Initialize BSP_ACCEL_INT_2 as input pin.

    GPIO_Digital_Input( &GPIOA_PDOR, _GPIO_PINMASK_17 );

    //  Initialize I2C module.

    I2C0_Init_Advanced( 100000, &_GPIO_Module_I2C0_PB2_3 );
}

#endif

// ============================================================================
//                                                                       WIFI
// ============================================================================

#if ( _MMB_MODULE_WIFI == 1 )

static volatile sbit CC3100_HIB_PIN     at          GPIOA_PDOR.B29;
static volatile sbit CC3100_CS_PIN      at          GPIOB_PDOR.B1;
static volatile sbit CC3100_RST_PIN     at          GPIOA_PDOR.B28;

const static hal_spi_obj_t BSP_WIFI_SPI_MODULE =
{
    SPI1_Write,
    SPI1_Read
};

static void hal_wifi_init ( void )
{
    //  Disable interrupt for WiFi module.

    NVIC_IntDisable( IVT_INT_PORTB );

    //  Initialize CC3100 HIB as output pin.

    GPIO_Digital_Output( &GPIOA_PDOR, _GPIO_PINMASK_29 );

    //  Initialize CC3100 RST as output pin.

    GPIO_Digital_Output( &GPIOA_PDOR, _GPIO_PINMASK_28 );

    //  Initialize CC3100 CS as output pin.

    GPIO_Digital_Output( &GPIOB_PDOR, _GPIO_PINMASK_1 );

    //  Initialize CC3100 INT as input pin.

    GPIO_Digital_Input( &GPIOB_PDIR, _GPIO_PINMASK_0 );

    /*
        Initialize CC3100 SPI peripheral.

        NOTE :

            SPI must be configured to SPI mode 0 ( IDLE LOW / FIRST EDGE ).
    */

    SPI1_Init_Advanced( 50000000,

                        _SPI_CFG_MASTER |
                        _SPI_CFG_FRAME_SIZE_8BITS |
                        _SPI_CFG_CLK_IDLE_LOW |
                        _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                        _SPI_CFG_MSB_FIRST,

                        &_GPIO_Module_SPI1_PB11_17_16 );
                        
    SPI_Set_Active( SPI1_Read, SPI1_Write );

    CC3100_CS_PIN = 1;
}

#endif

// ============================================================================
//                                                                     BUZZER
// ============================================================================

#if ( _MMB_MODULE_BUZZER == 1 )

static void hal_buzzer_init ( void )
{
    //  Initialize BUZZER pin as output.

    GPIO_Digital_Output( &GPIOE_PDOR, _GPIO_PINMASK_8 );

    //  Initialize Sound library.

    Sound_Init( &GPIOE_PDOR, 8 );
}

#endif

// ============================================================================
//                                                                        RTC
// ============================================================================

#if ( _MMB_MODULE_RTC == 1 )

static void hal_rtc_init ( void )
{
    //  Initialize RTC module.

    // SETTING RTC ACCESS
    SIM_SCGC6 = 0x20000000;
    Delay_100ms( );
    Delay_100ms( );
    
    // ENABLING RTC OSCILLATOR
    RTC_CR = 0x100;
    Delay_1sec( );
    
    // WRITE SECONDS TO TSR
    if(!RTC_TSR)
    {
       RTC_SR.TCE = 0;
       Delay_ms(200);
       RTC_TSR = 0x00;
    }
    
    // ENABLING TIMER COUNTER
    RTC_SR.TCE = 1;
    Delay_ms(200);
    
    // COUNTING STARTS
}

#endif

// ============================================================================
//                                                                        USB
// ============================================================================

#if ( _MMB_MODULE_USB == 1 )

//  Enables power for USB device.

static sbit BSP_USB_PSW at                          GPIOE_PDOR.B9;

static void hal_usb_init ( void )
{
    //  Enable alternate GPIO function if needed

//    GPIO_Alternate_Function_Enable( &_GPIO_MODULE_OTG_FS_PA89_10_11_12 );

    //  Initialize PWS pin as output.

    GPIO_Digital_Output( &GPIOE_PDOR, _GPIO_PINMASK_9 );

    // PSW to 0 to turn on power for device.

    BSP_USB_PSW = 1;
    Delay_ms(100);
    BSP_USB_PSW = 0;
}

#endif

// ============================================================================
//                                                                    SHUTTLE
// ============================================================================

#if ( _MMB_MODULE_SHUTTLE == 1 )



#endif
#endif

/* -------------------------------------------------------------------------- */
/*
    mmb_mcu_---.h

  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgment:
   This product includes software developed by the MikroElektonika.

4. Neither the name of the MikroElektonika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */