#include <stdint.h>

// STATUS Register
#define STATUS_00_REG         ( 0x00 )

// XYZ Data Registers
#define OUT_X_MSB_REG         ( 0x01 )
#define OUT_X_LSB_REG         ( 0x02 )
#define OUT_Y_MSB_REG         ( 0x03 )
#define OUT_Y_LSB_REG         ( 0x04 )
#define OUT_Z_MSB_REG         ( 0x05 )
#define OUT_Z_LSB_REG         ( 0x06 )

// WHO_AM_I Device ID Register
#define WHO_AM_I_REG          ( 0x0D )

//XYZ_DATA_CFG Sensor Data Configuration Register
#define XYZ_DATA_CFG_REG      ( 0x0E )

// FXOS_CTRL_REG1 System Control 1 Register
#define FXOS_CTRL_REG1        ( 0x2A )

// FXOS_CTRL_REG2 System Control 2 Register
#define FXOS_CTRL_REG2        ( 0x2B )

// FXOS_CTRL_REG3 Interrupt Control Register
#define FXOS_CTRL_REG3        ( 0x2C )

// FXOS_CTRL_REG4 Interrupt Enable Register
#define FXOS_CTRL_REG4        ( 0x2D )

// FXOS_CTRL_REG5 Interrupt Configuration Register
#define FXOS_CTRL_REG5        ( 0x2E )

// MAG XYZ Data Registers
#define M_OUT_X_MSB_REG       ( 0x33 )
#define M_OUT_X_LSB_REG       ( 0x34 )
#define M_OUT_Y_MSB_REG       ( 0x35 )
#define M_OUT_Y_LSB_REG       ( 0x36 )
#define M_OUT_Z_MSB_REG       ( 0x37 )
#define M_OUT_Z_LSB_REG       ( 0x38 )

// TEMP Registers
#define TEMP_REG              ( 0x51 )

// MAG FXOS_CTRL_REG1 System Control 1 Register
#define FXOS_M_CTRL_REG1      ( 0x5B )

// MAG FXOS_CTRL_REG2 System Control 2 Register
#define FXOS_M_CTRL_REG2      ( 0x5C )

// MAG FXOS_CTRL_REG3 System Control 3 Register
#define FXOS_M_CTRL_REG3      ( 0x5D )

void FXOS_WrSingle(unsigned short wAddr, unsigned short wData);
unsigned short FXOS_RdSingle(unsigned short rAddr);
void FXOS_RdSeq(unsigned short rAddr,
                        unsigned char *rdData,
                        unsigned long rLen);

// TFT module connections
sbit TFT_CS at GPIOB_PDOR.B20;
char TFT_DataPort at GPIOD_PDOR;
sbit TFT_RD at GPIOB_PDOR.B22;
sbit TFT_RS at GPIOB_PDOR.B21;
sbit TFT_RST at GPIOB_PDOR.B19;
sbit TFT_WR at GPIOB_PDOR.B23;
//sbit TFT_BLED at GPIOF_ODR.B10;

uint8_t data_[256];
uint8_t rData[28];
uint8_t txt01[4];
uint8_t txt11[7];
uint8_t txt2[7];
uint8_t txt3[7];
uint8_t id_reg;
int16_t sensorData[10];

void Init_Display() {
  TFT_Init_SSD1963_Board_50(800, 480);
  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
}

void FXOS_Init() {
  I2C0_Init_Advanced(100000, &_GPIO_MODULE_I2C0_PB2_3);
  FXOS_WrSingle(FXOS_CTRL_REG1, 0x00);               // Standby mode
  FXOS_WrSingle(FXOS_M_CTRL_REG1, 0x00);             //  Only accelerometer sensor is active
  FXOS_WrSingle(XYZ_DATA_CFG_REG, 0x02);             // full-scale range 8 g
  FXOS_WrSingle(FXOS_CTRL_REG1, 0x01);               // Active mode
}//~

//--------------- Writes data to FXOS8700CQ - signle location
void FXOS_WrSingle(unsigned short wAddr, unsigned short wData) {
  data_[0] = wAddr;
  data_[1] = wData;
  I2C0_Start();
  // issue I2C start signal
  I2C0_Write(0x1E, data_, 2, END_MODE_STOP);
}//~

//--------------- Reads data from FXOS8700CQ - single location (random)
unsigned short FXOS_RdSingle(unsigned short rAddr) {
  data_[0] = rAddr;
  I2C0_Start();              // issue I2C start signal
  I2C0_Write(0x1E, data_, 1, END_MODE_RESTART);
  I2C0_Read(0x1E, data_, 1, END_MODE_STOP);

  return data_[0];
}//~

//--------------- Reads data from FXOS8700CQ - sequential read

void FXOS_RdSeq(unsigned short rAddr,
                        unsigned char *rdData,
                        unsigned long rLen) {
  data_[0] = rAddr;
  I2C0_Start();              // issue I2C start signal
  I2C0_Write(0x1E,data_,1,END_MODE_RESTART);
  I2C0_Read(0x1E, rdData, rLen, END_MODE_STOP);
}//~

void main() {
   Init_Display();
   FXOS_Init();
   Delay_ms(100);
   TFT_Write_Text("ID = ", 65, 60);
   TFT_Write_Text("X = ", 65, 100);
   TFT_Write_Text("Y = ", 65, 130);
   TFT_Write_Text("Z = ", 65, 160);
   id_reg = FXOS_RdSingle(WHO_AM_I_REG);
   ByteToHex(id_reg, txt01);
   TFT_Write_Text(txt01, 95, 60);

SPI1_Init_Advanced(8000000, _SPI_CFG_MASTER | _SPI_CFG_FRAME_SIZE_8BITS | _SPI_CFG_CLK_IDLE_LOW | _SPI_CFG_CLK_PHASE_CAPT_LEADING | _SPI_CFG_MSB_FIRST, &_GPIO_Module_SPI1_PE2_3_1

   
   while (1) {
       FXOS_RdSeq(OUT_X_MSB_REG , rData, 6);
       Delay_ms(100);
       sensorData[0] = (int16_t)(( (int16_t)rData[0] << 8 | rData[1] ) >> 2);
       sensorData[1] = (int16_t)(( (int16_t)rData[2] << 8 | rData[3] ) >> 2);
       sensorData[2] = (int16_t)(( (int16_t)rData[4] << 8 | rData[5] ) >> 2);
       IntToStr(sensorData[0], txt11);
       IntToStr(sensorData[1], txt2);
       IntToStr(sensorData[2], txt3);
       Delay_ms(100);
       TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
       TFT_Set_Pen(CL_WHITE,1);
       TFT_Rectangle(85,100,200,190);
       TFT_Set_Font(TFT_defaultFont, CL_RED, FO_HORIZONTAL);
       TFT_Write_Text(txt1, 85, 100);
       TFT_Write_Text(txt2, 85, 130);
       TFT_Write_Text(txt3, 85, 160);
   }
}