/*
 * Project name:
     BUZZ click (Using mikroE's BUZZER)
 * Copyright:
     (c) Mikroelektronika, 2018.
 * Revision History:
     20120521:
       - initial release (JK and DO);
 * Description:
      A simple example of using BUZZER, which will play 
      a predefined pattern of sounds using Sound library :
      - "Scales"
      - "Ode To Joy"
 * Test configuration:
     MCU:             MK66FX1M0VLQ18

 */
 
#include "resources.h"
#include "notes.h"
 
#define SCALE_LENGTH 12    // define scale length
#define SCALE_COUNT   3    // define number os scales

// note intervals in miliseconds
#define FULL_NOTE         1200
#define HALF_DOT_NOTE      900
#define HALF_NOTE          600
#define QUARTER_DOT_NOTE   450
#define QUARTER_NOTE       300
#define EIGHTH_DOT_NOTE    225
#define EIGHTH_NOTE        150
#define SIXTINTH_DOT_NOTE  112
#define SIXTINTH_NOTE       75



// TFT module connections
sbit TFT_CS at GPIOB_PDOR.B20;
unsigned int TFT_DataPort at GPIOD_PDOR;
sbit TFT_RD at GPIOB_PDOR.B22;
sbit TFT_RS at GPIOB_PDOR.B21;
sbit TFT_RST at GPIOB_PDOR.B19;
sbit TFT_WR at GPIOB_PDOR.B23;
// End TFT module connections

void Init_Display() {
  TFT_Init_SSD1963_Board_50(800, 480);

  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
}
// Scale and note definitions
const unsigned int Scales[3][12] = {
 {523, 554, 587, 622, 659, 698, 740, 784, 831, 880, 932, 988},
 {1046, 1108, 1174, 1244, 1318, 1397, 1480,1568, 1662,1760, 1865, 1975},
 {2093, 2217, 2349, 2489, 2637, 2794, 2960, 3136, 3322, 3520, 3729, 3951}};

// The first several notes of famous Beethoven's tune
void Ode_to_Joy(){

  Sound_Play(Scales[0][7],  QUARTER_NOTE);       // G
  Sound_Play(Scales[0][7],  QUARTER_NOTE);       // G
  Sound_Play(Scales[0][8],  QUARTER_NOTE);       // A
  Sound_Play(Scales[0][10], QUARTER_NOTE);       // B
  Sound_Play(Scales[0][10], QUARTER_NOTE);       // B
  Sound_Play(Scales[0][8],  QUARTER_NOTE);       // A
  Sound_Play(Scales[0][7],  QUARTER_NOTE);       // G
  Sound_Play(Scales[0][5],  QUARTER_NOTE);       // F
  Sound_Play(Scales[0][3],  QUARTER_NOTE);       // D#
  Sound_Play(Scales[0][3],  QUARTER_NOTE);       // E
  Sound_Play(Scales[0][5],  QUARTER_NOTE);       // F
  Sound_Play(Scales[0][7],  QUARTER_NOTE);       // G
  Sound_Play(Scales[0][7],  QUARTER_DOT_NOTE);   // G.
  Sound_Play(Scales[0][5],  EIGHTH_NOTE);        // F
  Sound_Play(Scales[0][5],  QUARTER_DOT_NOTE);   // F.

}

// Play musical scale
void Test_scales(){
  unsigned short note = 0, scale = 0;

  for (scale = 0; scale < SCALE_COUNT; scale++)
    for (note = 0; note < SCALE_LENGTH; note++)
      Sound_Play(Scales[scale][note], 200);
}
 void buzz_melody()
{
    Sound_Play( _BUZZ_TONE_aHa, 500 );
    Sound_Play( _BUZZ_TONE_aHa, 500 );
    Sound_Play( _BUZZ_TONE_aHa, 500 );
    Sound_Play( _BUZZ_TONE_fS, 350 );
    Sound_Play( _BUZZ_TONE_cH, 150 );
    Sound_Play( _BUZZ_TONE_aHa, 500 );
    Sound_Play( _BUZZ_TONE_fS, 350 );
    Sound_Play( _BUZZ_TONE_cH, 150 );
    Sound_Play( _BUZZ_TONE_aHa, 650 );

    delay_ms( 500 );

    Sound_Play( _BUZZ_TONE_eH, 500 );
    Sound_Play( _BUZZ_TONE_eH, 500 );
    Sound_Play( _BUZZ_TONE_eH, 500 );
    Sound_Play( _BUZZ_TONE_fH, 350 );
    Sound_Play( _BUZZ_TONE_cH, 150 );
    Sound_Play( _BUZZ_TONE_gS, 500 );
    Sound_Play( _BUZZ_TONE_fS, 350 );
    Sound_Play( _BUZZ_TONE_cH, 150 );
    Sound_Play( _BUZZ_TONE_aHa, 650 );

    delay_ms( 500 );

    Sound_Play( _BUZZ_TONE_aH, 500 );
    Sound_Play( _BUZZ_TONE_aHa, 300 );
    Sound_Play( _BUZZ_TONE_aHa, 150 );
    Sound_Play( _BUZZ_TONE_aH, 500 );
    Sound_Play( _BUZZ_TONE_gSH, 325 );
    Sound_Play( _BUZZ_TONE_gH, 175 );
    Sound_Play( _BUZZ_TONE_fSH, 125 );
    Sound_Play( _BUZZ_TONE_fH, 125 );
    Sound_Play( _BUZZ_TONE_fSH, 250 );

    delay_ms( 325 );

    Sound_Play( _BUZZ_TONE_aS, 250 );
    Sound_Play( _BUZZ_TONE_dSH, 500 );
    Sound_Play( _BUZZ_TONE_dH, 325 );
    Sound_Play( _BUZZ_TONE_cSH, 175 );
    Sound_Play( _BUZZ_TONE_cH, 125 );
    Sound_Play( _BUZZ_TONE_b, 125 );
    Sound_Play( _BUZZ_TONE_cH, 250 );

    delay_ms( 350 );

    Sound_Play( _BUZZ_TONE_fS, 250 );
    Sound_Play( _BUZZ_TONE_gS, 500 );
    Sound_Play( _BUZZ_TONE_fS, 350 );
    Sound_Play( _BUZZ_TONE_aHa, 125 );
    Sound_Play( _BUZZ_TONE_cH, 500 );
    Sound_Play( _BUZZ_TONE_aHa, 375 );
    Sound_Play( _BUZZ_TONE_cH, 125 );
    Sound_Play( _BUZZ_TONE_eH, 650 );

    delay_ms( 500 );

    Sound_Play( _BUZZ_TONE_aH, 500 );
    Sound_Play( _BUZZ_TONE_aHa, 300 );
    Sound_Play( _BUZZ_TONE_aHa, 150 );
    Sound_Play( _BUZZ_TONE_aH, 500 );
    Sound_Play( _BUZZ_TONE_gSH, 325 );
    Sound_Play( _BUZZ_TONE_gH, 175 );
    Sound_Play( _BUZZ_TONE_fSH, 125 );
    Sound_Play( _BUZZ_TONE_fH, 125 );
    Sound_Play( _BUZZ_TONE_fSH, 250 );

    delay_ms( 325 );

    Sound_Play( _BUZZ_TONE_aS, 250 );
    Sound_Play( _BUZZ_TONE_dSH, 500 );
    Sound_Play( _BUZZ_TONE_dH, 325 );
    Sound_Play( _BUZZ_TONE_cSH, 175 );
    Sound_Play( _BUZZ_TONE_cH, 125 );
    Sound_Play( _BUZZ_TONE_b, 125 );
    Sound_Play( _BUZZ_TONE_cH, 250 );

    delay_ms( 350 );

    Sound_Play( _BUZZ_TONE_fS, 250 );
    Sound_Play( _BUZZ_TONE_gS, 500 );
    Sound_Play( _BUZZ_TONE_fS, 375 );
    Sound_Play( _BUZZ_TONE_cH, 125 );
    Sound_Play( _BUZZ_TONE_aHa, 500 );
    Sound_Play( _BUZZ_TONE_fS, 375 );
    Sound_Play( _BUZZ_TONE_cH, 125 );
    Sound_Play( _BUZZ_TONE_aHa, 650 );

    delay_ms( 650 );

}

void main() {
  GPIO_Digital_Output(&GPIOE_PDOR, _GPIO_PINMASK_8);                    // Set PB8 as digital output

  Init_Display();

  Sound_Init(&GPIOE_PDOR, 8);                    // Initialize Sound library

  TFT_Fill_Screen(CL_WHITE);                     // Clear Screen

  while (1){
      TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
      TFT_Set_Pen(CL_WHITE,1);
      TFT_Rectangle(65,90,200,120);
      TFT_Write_Text("Playing Musical Scales", 65, 90);
      Test_scales();
      Delay_100ms();
      TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
      TFT_Set_Pen(CL_WHITE,1);
      TFT_Rectangle(65,90,200,120);
      TFT_Write_Text("Playing Ode To Joy", 65, 90);
      Ode_to_Joy();
      Delay_100ms();
      TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
      TFT_Set_Pen(CL_WHITE,1);
      TFT_Rectangle(65,90,200,120);
      TFT_Write_Text("Playing Imperial March", 65, 90);
      buzz_melody();
  }
}