/*********************************************************************************
  Library: GPIO
  Family:  MK64
**********************************************************************************/

#include "__Lib_GPIO.h"
// Port/Pin control register offsets. First byte in pin register address need to be cleared
// 0xXXXXXX00 + offset
#define _PORT_GPCLR_OFFSET      0x80
#define _PORT_GPCHR_OFFSET      0x84
#define _PORT_ISFR_OFFSET       0xA0
#define _PORT_DFER_OFFSET       0xC0
#define _PORT_DFCR_OFFSET       0xC4
#define _PORT_DFWR_OFFSET       0xC8

// GPIO Port Register offsets
#define _GPIO_PORT_PDOR_OFFSET  0x00
#define _GPIO_PORT_PSOR_OFFSET  0x04/4
#define _GPIO_PORT_PCOR_OFFSET  0x08/4
#define _GPIO_PORT_PTOR_OFFSET  0x0C/4
#define _GPIO_PORT_PDIR_OFFSET  0x10/4
#define _GPIO_PORT_PDDR_OFFSET  0x14/4

// Pin masks
////////////
const unsigned long
  _GPIO_PINMASK_0    = 0x00000001,
  _GPIO_PINMASK_1    = 0x00000002,
  _GPIO_PINMASK_2    = 0x00000004,
  _GPIO_PINMASK_3    = 0x00000008,
  _GPIO_PINMASK_4    = 0x00000010,
  _GPIO_PINMASK_5    = 0x00000020,
  _GPIO_PINMASK_6    = 0x00000040,
  _GPIO_PINMASK_7    = 0x00000080,
  _GPIO_PINMASK_8    = 0x00000100,
  _GPIO_PINMASK_9    = 0x00000200,
  _GPIO_PINMASK_10   = 0x00000400,
  _GPIO_PINMASK_11   = 0x00000800,
  _GPIO_PINMASK_12   = 0x00001000,
  _GPIO_PINMASK_13   = 0x00002000,
  _GPIO_PINMASK_14   = 0x00004000,
  _GPIO_PINMASK_15   = 0x00008000,
  _GPIO_PINMASK_16   = 0x00010000,
  _GPIO_PINMASK_17   = 0x00020000,
  _GPIO_PINMASK_18   = 0x00040000,
  _GPIO_PINMASK_19   = 0x00080000,
  _GPIO_PINMASK_20   = 0x00100000,
  _GPIO_PINMASK_21   = 0x00200000,
  _GPIO_PINMASK_22   = 0x00400000,
  _GPIO_PINMASK_23   = 0x00800000,
  _GPIO_PINMASK_24   = 0x01000000,
  _GPIO_PINMASK_25   = 0x02000000,
  _GPIO_PINMASK_26   = 0x04000000,
  _GPIO_PINMASK_27   = 0x08000000,
  _GPIO_PINMASK_28   = 0x10000000,
  _GPIO_PINMASK_29   = 0x20000000,
  _GPIO_PINMASK_30   = 0x40000000,
  _GPIO_PINMASK_31   = 0x80000000,
  
  _GPIO_PINMASK_LOW     = 0x000000FF,
  _GPIO_PINMASK_HIGH    = 0x0000FF00,
  _GPIO_PINMASK_HIGHER  = 0x00FF0000,
  _GPIO_PINMASK_HIGHEST = 0xFF000000,
  _GPIO_PINMASK_ALL     = 0xFFFFFFFF;

// Pin numbers
//////////////
const
  _GPIO_PIN_0  = 0,
  _GPIO_PIN_1  = 1,
  _GPIO_PIN_2  = 2,
  _GPIO_PIN_3  = 3,
  _GPIO_PIN_4  = 4,
  _GPIO_PIN_5  = 5,
  _GPIO_PIN_6  = 6,
  _GPIO_PIN_7  = 7,
  _GPIO_PIN_8  = 8,
  _GPIO_PIN_9  = 9,
  _GPIO_PIN_10 = 10,
  _GPIO_PIN_11 = 11,
  _GPIO_PIN_12 = 12,
  _GPIO_PIN_13 = 13,
  _GPIO_PIN_14 = 14,
  _GPIO_PIN_15 = 15,
  _GPIO_PIN_16 = 16,
  _GPIO_PIN_17 = 17,
  _GPIO_PIN_18 = 18,
  _GPIO_PIN_19 = 19,
  _GPIO_PIN_20 = 20,
  _GPIO_PIN_21 = 21,
  _GPIO_PIN_22 = 22,
  _GPIO_PIN_23 = 23,
  _GPIO_PIN_24 = 24,
  _GPIO_PIN_25 = 25,
  _GPIO_PIN_26 = 26,
  _GPIO_PIN_27 = 27,
  _GPIO_PIN_28 = 28,
  _GPIO_PIN_29 = 29,
  _GPIO_PIN_30 = 30,
  _GPIO_PIN_31 = 31;
  
// Pin Control (PCR) register mask
const unsigned long
  // Pull Select
  _GPIO_CFG_PULL_UP                             = 0x1ul,
  _GPIO_CFG_PULL_DOWN                           = 0x0,

  // Pull Enable
  _GPIO_CFG_PULL_ENABLE                         = 0x2ul,
  _GPIO_CFG_PULL_DISABLE                        = 0x0,

  // Slew Rate Enable
  _GPIO_CFG_SLEW_RATE_SLOW                      = 0x4ul,
  _GPIO_CFG_SLEW_RATE_FAST                      = 0x0,

  // Passive Filter Enable
  _GPIO_CFG_PASSIVE_FILT_ENABLE                 = 0x10ul,
  _GPIO_CFG_PASSIVE_FILT_DISABLE                = 0x0,

  // Open Drain Enable
  _GPIO_CFG_TYPE_OD_ENABLED                     = 0x20ul,
  _GPIO_CFG_TYPE_OD_DISABLED                    = 0x0,

  // Drive Strength Enable
  _GPIO_CFG_DRIVE_STRENGTH_HIGH                 = 0x40ul,
  _GPIO_CFG_DRIVE_STRENGTH_LOW                  = 0x0;

static const unsigned long
  _GPIO_CFG_PCR_PS_MASK                         = 0x1ul,
  _GPIO_CFG_PCR_PS_SHIFT                        = 0,
  
  _GPIO_CFG_PCR_PE_MASK                         = 0x2ul,
  _GPIO_CFG_PCR_PE_SHIFT                        = 1,
  
  _GPIO_CFG_PCR_SRE_MASK                        = 0x4ul,
  _GPIO_CFG_PCR_SRE_SHIFT                       = 2,
  
  _GPIO_CFG_PCR_PFE_MASK                        = 0x10ul,
  _GPIO_CFG_PCR_PFE_SHIFT                       = 4,
 
  _GPIO_CFG_PCR_ODE_MASK                        = 0x20ul,
  _GPIO_CFG_PCR_ODE_SHIFT                       = 5,
  
  _GPIO_CFG_PCR_DSE_MASK                        = 0x40ul,
  _GPIO_CFG_PCR_DSE_SHIFT                       = 6,
  // Pin Mux Control
  _GPIO_CFG_PCR_MUX_MASK                        = 0x700ul,
  _GPIO_CFG_PCR_MUX_SHIFT                       = 8,
  // Lock Register
  _GPIO_CFG_PCR_LK_MASK                         = 0x8000ul,
  _GPIO_CFG_PCR_LK_SHIFT                        = 15;

const unsigned long
  // mode
  _GPIO_CFG_MODE_ANALOG = 0x10000,
  _GPIO_CFG_MODE_INPUT = 0x20000,
  _GPIO_CFG_MODE_OUTPUT = 0x40000,
  _GPIO_CFG_MODE_ALT_FUNCTION = 0x80000;

const unsigned long
// Pin Mux Alternate Function
  _GPIO_CFG_MUX_ALT0                        = 0x0ul,
  _GPIO_CFG_MUX_ALT1                        = 0x100ul,
  _GPIO_CFG_MUX_ALT2                        = 0x200ul,
  _GPIO_CFG_MUX_ALT3                        = 0x300ul,
  _GPIO_CFG_MUX_ALT4                        = 0x400ul,
  _GPIO_CFG_MUX_ALT5                        = 0x500ul,
  _GPIO_CFG_MUX_ALT6                        = 0x600ul,
  _GPIO_CFG_MUX_ALT7                        = 0x700ul;
  
static const unsigned long
  _GPIO_PORTA                               = 0,
  _GPIO_PORTB                               = 1,
  _GPIO_PORTC                               = 2,
  _GPIO_PORTD                               = 3,
  _GPIO_PORTE                               = 4;

// Digital/analog pin configuration
///////////////////////////////////
const unsigned long
  // all digital signals are at ALT1
  // all adc signals are on ALT0
  _GPIO_CFG_DIGITAL_OUTPUT = _GPIO_CFG_MODE_OUTPUT | _GPIO_CFG_PULL_DISABLE | _GPIO_CFG_SLEW_RATE_SLOW | _GPIO_CFG_PASSIVE_FILT_DISABLE | 
                             _GPIO_CFG_TYPE_OD_DISABLED | _GPIO_CFG_DRIVE_STRENGTH_LOW | _GPIO_CFG_MUX_ALT1,
  _GPIO_CFG_DIGITAL_INPUT  = _GPIO_CFG_MODE_INPUT | _GPIO_CFG_PULL_DISABLE | _GPIO_CFG_SLEW_RATE_SLOW | _GPIO_CFG_PASSIVE_FILT_DISABLE |
                             _GPIO_CFG_DRIVE_STRENGTH_LOW | _GPIO_CFG_MUX_ALT1,
  _GPIO_CFG_ANALOG    = _GPIO_CFG_MODE_ANALOG | _GPIO_CFG_MUX_ALT0;

#define _GPIO_PORT_CTRL_OFFSET     0x40
#define _GPIO_PORT_CTRL_MASK       0x1FF
#define _GPIO_PORT_CTRL_BASE       0x400FF000
#define _PIN_CTRL_BASE             0x40049000
#define _PIN_CTRL_OFFSET           0x1000

// Masks' offsets
static const unsigned long
  _GPIO_MUX_OFFSET  = 8,
  _GPIO_PORT_OFFSET = 5;
  
const unsigned long
  _GPIO_CFG_MUX2_SPI  = (2ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX3_UART = (3ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX7_SPI  = (7ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX2_I2C  = (2ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX4_I2C  = (4ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX5_I2C  = (5ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX6_I2C  = (6ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX7_I2C  = (7ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX2_CAN  = (2ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX4_SDHC = (4ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX3_PWM  = (3ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX4_PWM  = (4ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX7_PWM  = (7ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX6_PWM  = (6ul << _GPIO_MUX_OFFSET),
  _GPIO_CFG_MUX4_ETH  = (4ul << _GPIO_MUX_OFFSET);

static const unsigned long
  _GPIO_MUX2_SPI      = 2ul,
  _GPIO_MUX3_UART     = 3ul,
  _GPIO_MUX7_SPI      = 7ul,
  _GPIO_MUX2_I2C      = 2ul,
  _GPIO_MUX4_I2C      = 4ul,
  _GPIO_MUX5_I2C      = 5ul,
  _GPIO_MUX6_I2C      = 6ul,
  _GPIO_MUX7_I2C      = 7ul,
  _GPIO_MUX4_SDHC     = 4ul,
  _GPIO_MUX3_PWM      = 3ul,
  _GPIO_MUX4_PWM      = 4ul,
  _GPIO_MUX4_ETH      = 4ul,
  _GPIO_CFG_MUX2_ETH  = (2ul << _GPIO_MUX_OFFSET);
  
const unsigned long
  _USART_CONFIG = _GPIO_CFG_MODE_ALT_FUNCTION,
  _SPI_CONFIG   = _GPIO_CFG_MODE_ALT_FUNCTION,
  _I2C_CONFIG   = _GPIO_CFG_MODE_ALT_FUNCTION | _GPIO_CFG_TYPE_OD_ENABLED,
  _CAN_CONFIG   = _GPIO_CFG_MODE_ALT_FUNCTION,
  _SDHC_CONFIG  = _GPIO_CFG_MODE_ALT_FUNCTION | _GPIO_CFG_PULL_ENABLE | _GPIO_CFG_PULL_UP | _GPIO_CFG_DRIVE_STRENGTH_HIGH,
  _PWM_CONFIG   = _GPIO_CFG_MODE_ALT_FUNCTION,
  _ETH_CONFIG   = _GPIO_CFG_MODE_ALT_FUNCTION;
  
unsigned long const
  //// UART ////
  _GPIO_UART0_RX_PB16 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (16),
  _GPIO_UART0_TX_PB17 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (17),

  _GPIO_UART1_TX_PE0 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (0),
  _GPIO_UART1_RX_PE1 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (1),

  _GPIO_UART1_TX_PC4 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (4),
  _GPIO_UART1_RX_PC3 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (3),
  
  _GPIO_UART2_TX_PD3 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (3),
  _GPIO_UART2_RX_PD2 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (2),

  _GPIO_UART2_TX_PE16 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (16),
  _GPIO_UART2_RX_PE17 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (17),
  
  _GPIO_UART3_RX_PC16 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (16),
  _GPIO_UART3_TX_PC17 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (17),

  _GPIO_UART4_TX_PE24 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (24),
  _GPIO_UART4_RX_PE25 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (25),
  
  _GPIO_UART5_TX_PE8 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (8),
  _GPIO_UART5_RX_PE9 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (9),
  
  _GPIO_UART5_TX_PD9 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (9),
  _GPIO_UART5_RX_PD8 = _GPIO_CFG_MUX3_UART | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (8),
  
  //// SPI ////
  _GPIO_SPI0_SCK_PTA15  = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (15),
  _GPIO_SPI0_MOSI_PTA16 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (16),
  _GPIO_SPI0_MISO_PTA17 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (17),
  
  _GPIO_SPI0_SCK_PTC5  = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (5),
  _GPIO_SPI0_MOSI_PTC6 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (6),
  _GPIO_SPI0_MISO_PTC7 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (7),
  
  _GPIO_SPI0_SCK_PTD1  = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (1),
  _GPIO_SPI0_MOSI_PTD2 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (2),
  _GPIO_SPI0_MISO_PTD3 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (3),
  
  _GPIO_SPI1_SCK_PTE2   = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (2),
  _GPIO_SPI1_MOSI_PTE1  = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (1),
  _GPIO_SPI1_MISO_PTE3  = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (3),
  
  _GPIO_SPI1_MISO_PTE1  = _GPIO_CFG_MUX7_SPI | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (1),
  _GPIO_SPI1_MOSI_PTE3  = _GPIO_CFG_MUX7_SPI | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (3),

  _GPIO_SPI1_SCK_PTB11  = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (11),
  _GPIO_SPI1_MOSI_PTB16 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (16),
  _GPIO_SPI1_MISO_PTB17 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (17),
  
  _GPIO_SPI1_SCK_PTD5   = _GPIO_CFG_MUX7_SPI | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (5),
  _GPIO_SPI1_MOSI_PTD6  = _GPIO_CFG_MUX7_SPI | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (6),
  _GPIO_SPI1_MISO_PTD7  = _GPIO_CFG_MUX7_SPI | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (7),

  _GPIO_SPI2_SCK_PTB21  = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (21),
  _GPIO_SPI2_MOSI_PTB22 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (22),
  _GPIO_SPI2_MISO_PTB23 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (23),

  _GPIO_SPI2_SCK_PTD12  = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (12),
  _GPIO_SPI2_MOSI_PTD13 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (13),
  _GPIO_SPI2_MISO_PTD14 = _GPIO_CFG_MUX2_SPI | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (14),

  //// I2C ////
  _GPIO_I2C0_SCL_PTE24 = _GPIO_CFG_MUX5_I2C | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (24),
  _GPIO_I2C0_SDA_PTE25 = _GPIO_CFG_MUX5_I2C | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (25),
  
  _GPIO_I2C0_SCL_PTB0 = _GPIO_CFG_MUX2_I2C | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (0),
  _GPIO_I2C0_SDA_PTB1 = _GPIO_CFG_MUX2_I2C | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (1),

  _GPIO_I2C0_SCL_PTB2 = _GPIO_CFG_MUX2_I2C | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (2),
  _GPIO_I2C0_SDA_PTB3 = _GPIO_CFG_MUX2_I2C | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (3),

  _GPIO_I2C0_SCL_PTD2 = _GPIO_CFG_MUX7_I2C | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (2),
  _GPIO_I2C0_SDA_PTD3 = _GPIO_CFG_MUX7_I2C | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (3),

  _GPIO_I2C0_SCL_PTD8 = _GPIO_CFG_MUX2_I2C | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (8),
  _GPIO_I2C0_SDA_PTD9 = _GPIO_CFG_MUX2_I2C | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (9),  
  
  _GPIO_I2C1_SCL_PTE1 = _GPIO_CFG_MUX6_I2C | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (1),
  _GPIO_I2C1_SDA_PTE0 = _GPIO_CFG_MUX6_I2C | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (0),

  _GPIO_I2C1_SCL_PTC10 = _GPIO_CFG_MUX2_I2C | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (10),
  _GPIO_I2C1_SDA_PTC11 = _GPIO_CFG_MUX2_I2C | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (11),
  
  _GPIO_I2C2_SCL_PTA12 = _GPIO_CFG_MUX5_I2C | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (12),
  _GPIO_I2C2_SDA_PTA11 = _GPIO_CFG_MUX5_I2C | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (11),
  
  _GPIO_I2C2_SCL_PTA14 = _GPIO_CFG_MUX5_I2C | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (14),
  _GPIO_I2C2_SDA_PTA13 = _GPIO_CFG_MUX5_I2C | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (13),
  
  _GPIO_I2C3_SCL_PTE11 = _GPIO_CFG_MUX2_I2C | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (11),
  _GPIO_I2C3_SDA_PTE10 = _GPIO_CFG_MUX2_I2C | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (10),
  
  _GPIO_I2C3_SCL_PTA2 = _GPIO_CFG_MUX4_I2C | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (2),
  _GPIO_I2C3_SDA_PTA1 = _GPIO_CFG_MUX4_I2C | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (1),
  
  //// CAN ////
  _GPIO_CAN0_TX_PTA12 = _GPIO_CFG_MUX2_CAN | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (12),
  _GPIO_CAN0_RX_PTA13 = _GPIO_CFG_MUX2_CAN | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (13),
  
  _GPIO_CAN0_RX_PTB19 = _GPIO_CFG_MUX2_CAN | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (19),
  _GPIO_CAN0_TX_PTB18 = _GPIO_CFG_MUX2_CAN | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (18),
  
  _GPIO_CAN1_TX_PTE24 = _GPIO_CFG_MUX2_CAN | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (24),
  _GPIO_CAN1_RX_PTE25 = _GPIO_CFG_MUX2_CAN | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (25),

  _GPIO_CAN1_RX_PTC16 = _GPIO_CFG_MUX2_CAN | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (16),
  _GPIO_CAN1_TX_PTC17 = _GPIO_CFG_MUX2_CAN | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (17),
  
  
  //// SDHC ////
  _GPIO_SDHC_D0_PTE1     = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (1),
  _GPIO_SDHC_D1_PTE0     = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (0),
  _GPIO_SDHC_D2_PTE5     = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (5),
  _GPIO_SDHC_D3_PTE4     = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (4),
  _GPIO_SDHC_D4_PTD12    = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (12),
  _GPIO_SDHC_D5_PTD13    = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (13),
  _GPIO_SDHC_D6_PTD14    = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (14),
  _GPIO_SDHC_D7_PTD15    = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (15),
  _GPIO_SDHC_DCLK_PTE2   = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (2),
  _GPIO_SDHC_CMD_PTE3    = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTE << _GPIO_PORT_OFFSET) | (3),
  _GPIO_SDHC_CLKIN_PTD11 = _GPIO_CFG_MUX4_SDHC | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (11),
  
  _GPIO_FTM0_CH0_PTA3 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (3),
  _GPIO_FTM0_CH0_PTC1 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (1),
  _GPIO_FTM0_CH1_PTA4 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (4),
  _GPIO_FTM0_CH1_PTC2 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (2),
  _GPIO_FTM0_CH2_PTC3 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (3),
  _GPIO_FTM0_CH2_PTC5 = _GPIO_CFG_MUX7_PWM | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (5),
  _GPIO_FTM0_CH2_PTA5 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (5),
  _GPIO_FTM0_CH3_PTA6 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (6),
  _GPIO_FTM0_CH3_PTC4 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTC << _GPIO_PORT_OFFSET) | (4),
  _GPIO_FTM0_CH4_PTA7 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (7),
  _GPIO_FTM0_CH4_PTD4 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (4),
  _GPIO_FTMO_CH4_PTB12 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (12),
  _GPIO_FTM0_CH5_PTB13 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (13),
  _GPIO_FTM0_CH5_PTA0 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (0),
  _GPIO_FTM0_CH5_PTD5 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (5),
  _GPIO_FTM0_CH6_PTA1 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (1),
  _GPIO_FTM0_CH6_PTD6 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (6),
  _GPIO_FTM0_CH7_PTA2 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (2),
  _GPIO_FTM0_CH7_PTD7 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTD << _GPIO_PORT_OFFSET) | (7),

  _GPIO_FTM1_CH0_PTB12 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTB << _GPIO_PORT_OFFSET) | (12),
  _GPIO_FTM1_CH0_PTA8 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (8),
  _GPIO_FTM1_CH0_PTA12 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) | (12),
  _GPIO_FTM1_CH0_PTB0 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(0),

  _GPIO_FTM1_CH1_PTB13 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(13),
  _GPIO_FTM1_CH1_PTA9 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(9),
  _GPIO_FTM1_CH1_PTA13 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(13),
  _GPIO_FTM1_CH1_PTB1 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(1),

  _GPIO_FTM2_CH0_PTA10 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET)|(10),
  _GPIO_FTM2_CH0_PTB18 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(18),
  _GPIO_FTM2_CH1_PTA11 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(11),
  _GPIO_FTM2_CH1_PTB19 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(19),

  _GPIO_FTM3_CH0_PTE5 = _GPIO_CFG_MUX6_PWM | (_GPIO_PORTE << _GPIO_PORT_OFFSET) |(5),
  _GPIO_FTM3_CH0_PTD0 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTD << _GPIO_PORT_OFFSET) |(0),
  _GPIO_FTM3_CH1_PTE6 = _GPIO_CFG_MUX6_PWM | (_GPIO_PORTE << _GPIO_PORT_OFFSET) |(6),
  _GPIO_FTM3_CH1_PTD1 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTD << _GPIO_PORT_OFFSET) |(1),
  _GPIO_FTM3_CH2_PTE7 = _GPIO_CFG_MUX6_PWM | (_GPIO_PORTE << _GPIO_PORT_OFFSET) |(7),
  _GPIO_FTM3_CH2_PTD2 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTD << _GPIO_PORT_OFFSET) |(2),
  _GPIO_FTM3_CH3_PTE8 = _GPIO_CFG_MUX6_PWM | (_GPIO_PORTE << _GPIO_PORT_OFFSET) |(8),
  _GPIO_FTM3_CH3_PTD3 = _GPIO_CFG_MUX4_PWM | (_GPIO_PORTD << _GPIO_PORT_OFFSET) |(3),
  _GPIO_FTM3_CH4_PTE9 = _GPIO_CFG_MUX6_PWM | (_GPIO_PORTE << _GPIO_PORT_OFFSET) |(9),
  _GPIO_FTM3_CH4_PTC8 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTC << _GPIO_PORT_OFFSET) |(8),
  _GPIO_FTM3_CH5_PTE10 = _GPIO_CFG_MUX6_PWM | (_GPIO_PORTE << _GPIO_PORT_OFFSET) |(10),
  _GPIO_FTM3_CH5_PTC9 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTC << _GPIO_PORT_OFFSET) |(9),
  _GPIO_FTM3_CH6_PTE11 = _GPIO_CFG_MUX6_PWM | (_GPIO_PORTE << _GPIO_PORT_OFFSET) |(11),
  _GPIO_FTM3_CH6_PTC10 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTC << _GPIO_PORT_OFFSET) |(10),
  _GPIO_FTM3_CH7_PTE12 = _GPIO_CFG_MUX6_PWM | (_GPIO_PORTE << _GPIO_PORT_OFFSET) |(12),
  _GPIO_FTM3_CH7_PTC11 = _GPIO_CFG_MUX3_PWM | (_GPIO_PORTC << _GPIO_PORT_OFFSET) |(11),
  
  //// Ethernet ////
  _GPIO_ETH_TXD1_PTA17      = _GPIO_CFG_MUX_ALT4 | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(17),
  _GPIO_ETH_TXD0_PTA16      = _GPIO_CFG_MUX_ALT4 | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(16),
  _GPIO_ETH_TXEN_PTA15      = _GPIO_CFG_MUX_ALT4 | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(15),
  _GPIO_ETH_CLKIN_PTE26     = _GPIO_CFG_MUX_ALT2 | (_GPIO_PORTE << _GPIO_PORT_OFFSET) |(26),
  _GPIO_ETH_MDC_PTA8        = _GPIO_CFG_MUX_ALT5 | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(8),
  _GPIO_ETH_MDIO_PTA7       = _GPIO_CFG_MUX_ALT5 | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(7),
  _GPIO_ETH_CRS_DV_PTA14    = _GPIO_CFG_MUX_ALT4 | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(14),
  _GPIO_ETH_RXER_PTA5       = _GPIO_CFG_MUX_ALT4 | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(5),
  _GPIO_ETH_RXD0_PTA13      = _GPIO_CFG_MUX_ALT4 | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(13),
  _GPIO_ETH_RXD1_PTA12      = _GPIO_CFG_MUX_ALT4 | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(12),
  _GPIO_ETH_TXER_PTA28      = _GPIO_CFG_MUX_ALT4 | (_GPIO_PORTA << _GPIO_PORT_OFFSET) |(28),
  _GPIO_ETH_MDC_PTB1        = _GPIO_CFG_MUX_ALT4 | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(1),
  _GPIO_ETH_MDIO_PTB0       = _GPIO_CFG_MUX_ALT4 | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(0),
  _GPIO_ETH_1558_TMR0_PTB2 = _GPIO_CFG_MUX4_ETH | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(2),
  _GPIO_ETH_1558_TMR1_PTB3 = _GPIO_CFG_MUX4_ETH | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(3),
  _GPIO_ETH_1558_TMR2_PTB4 = _GPIO_CFG_MUX4_ETH | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(4),
  _GPIO_ETH_1558_TMR3_PTB5 = _GPIO_CFG_MUX4_ETH | (_GPIO_PORTB << _GPIO_PORT_OFFSET) |(5),
  _GPIO_ETH_1558_TMR0_PTC16 = _GPIO_CFG_MUX4_ETH | (_GPIO_PORTC << _GPIO_PORT_OFFSET) |(16),
  _GPIO_ETH_1558_TMR1_PTC17 = _GPIO_CFG_MUX4_ETH | (_GPIO_PORTC << _GPIO_PORT_OFFSET) |(17),
  _GPIO_ETH_1558_TMR2_PTC18 = _GPIO_CFG_MUX4_ETH | (_GPIO_PORTC << _GPIO_PORT_OFFSET) |(18),
  _GPIO_ETH_1558_TMR3_PTC19 = _GPIO_CFG_MUX4_ETH | (_GPIO_PORTC << _GPIO_PORT_OFFSET) |(19);
  
const Module_Struct
  //// UART ////
  _GPIO_Module_UART0_PB16_17 = {
    {_GPIO_UART0_TX_PB17, _GPIO_UART0_RX_PB16, -1},
    {_USART_CONFIG, _USART_CONFIG, -1}
  },
  
  _GPIO_Module_UART1_PE0_1 = {
    {_GPIO_UART1_TX_PE0, _GPIO_UART1_RX_PE1, -1},
    {_USART_CONFIG, _USART_CONFIG, -1}
  },

  _GPIO_Module_UART1_PC4_3 = {
    {_GPIO_UART1_TX_PC4, _GPIO_UART1_RX_PC3, -1},
    {_USART_CONFIG, _USART_CONFIG, -1}
  },
  
  _GPIO_Module_UART2_PD3_2 = {
    {_GPIO_UART2_TX_PD3, _GPIO_UART2_RX_PD2, -1},
    {_USART_CONFIG, _USART_CONFIG, -1}
  },

  _GPIO_Module_UART2_PE16_17 = {
    {_GPIO_UART2_TX_PE16, _GPIO_UART2_RX_PE17, -1},
    {_USART_CONFIG, _USART_CONFIG, -1}
  },

  _GPIO_Module_UART3_PC16_17 = {
    {_GPIO_UART3_TX_PC17, _GPIO_UART3_RX_PC16, -1},
    {_USART_CONFIG, _USART_CONFIG, -1}
  },
  
  _GPIO_Module_UART4_PE24_25 = {
    {_GPIO_UART4_TX_PE24, _GPIO_UART4_RX_PE25, -1},
    {_USART_CONFIG, _USART_CONFIG, -1}
  },
  
  _GPIO_Module_UART5_PE8_9 = {
    {_GPIO_UART5_TX_PE8, _GPIO_UART5_RX_PE9, -1},
    {_USART_CONFIG, _USART_CONFIG, -1}
  },
  
  _GPIO_Module_UART5_PD9_8 = {
    {_GPIO_UART5_TX_PD9, _GPIO_UART5_RX_PD8, -1},
    {_USART_CONFIG, _USART_CONFIG, -1}
  },
  
 _GPIO_Module_PWM0_PTA3 = {
    {  _GPIO_FTM0_CH0_PTA3, -1, -1},
    { _PWM_CONFIG, -1, -1}
    },

   _GPIO_Module_PWM0_PTC1 = {
   { _GPIO_FTM0_CH0_PTC1, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM0_PTA4 = {
   { _GPIO_FTM0_CH1_PTA4, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM0_PTC2 = {
   { _GPIO_FTM0_CH1_PTC2, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM0_PTC3 = {
   { _GPIO_FTM0_CH2_PTC3, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
  
  _GPIO_Module_PWM0_PTC5 = {
   { _GPIO_FTM0_CH2_PTC5, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
  
  _GPIO_Module_PWM0_PTA5 = {
   { _GPIO_FTM0_CH2_PTA5, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   _GPIO_Module_PWM0_PTA6 = {
   { _GPIO_FTM0_CH3_PTA6, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
     _GPIO_Module_PWM0_PTC4 = {
   { _GPIO_FTM0_CH3_PTC4, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM0_PTA7 = {
   { _GPIO_FTM0_CH4_PTA7, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM0_PTD4 = {
   { _GPIO_FTM0_CH4_PTD4, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },

   
   _GPIO_Module_PWM0_PTB12 = {
   { _GPIO_FTMO_CH4_PTB12, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   
   _GPIO_Module_PWM0_PTB13 = {
   { _GPIO_FTM0_CH5_PTB13, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },

    _GPIO_Module_PWM0_PTA0 = {
   { _GPIO_FTM0_CH5_PTA0, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM0_PTD5 = {
   { _GPIO_FTM0_CH5_PTD5, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
  
   _GPIO_Module_PWM0_PTA1 = {
   { _GPIO_FTM0_CH6_PTA1, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM0_PTD6 = {
   { _GPIO_FTM0_CH6_PTD6, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM0_PTA2 = {
   { _GPIO_FTM0_CH7_PTA2, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM0_PTD7 = {
   { _GPIO_FTM0_CH7_PTD7, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },

    _GPIO_Module_PWM1_PTB12 = {
   { _GPIO_FTM1_CH0_PTB12, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },

    _GPIO_Module_PWM1_PTA8 = {
   { _GPIO_FTM1_CH0_PTA8, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM1_PTA12 = {
   { _GPIO_FTM1_CH0_PTA12, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
  
    _GPIO_Module_PWM1_PTB0 = {
   { _GPIO_FTM1_CH0_PTB0, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
     _GPIO_Module_PWM1_PTB13 = {
   { _GPIO_FTM1_CH1_PTB13, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
  
    _GPIO_Module_PWM1_PTA9 = {
   { _GPIO_FTM1_CH1_PTA9, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },

   _GPIO_Module_PWM1_PTA13 = {
   { _GPIO_FTM1_CH1_PTA13, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM1_PTB1 = {
   { _GPIO_FTM1_CH1_PTB1, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
    _GPIO_Module_PWM2_PTA10 = {
   { _GPIO_FTM2_CH0_PTA10, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
     _GPIO_Module_PWM2_PTB18 = {
   { _GPIO_FTM2_CH0_PTB18, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM2_PTA11 = {
   { _GPIO_FTM2_CH1_PTA11, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM2_PTB19 = {
   { _GPIO_FTM2_CH1_PTB19, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM3_PTE5 = {
   { _GPIO_FTM3_CH0_PTE5, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM3_PTD0 = {
   { _GPIO_FTM3_CH0_PTD0, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
    _GPIO_Module_PWM3_PTE6 = {
   { _GPIO_FTM3_CH1_PTE6, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   //_GPIO_FTM3_CH1_PTD1
   _GPIO_Module_PWM3_PTD1 = {
   { _GPIO_FTM3_CH1_PTD1, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM3_PTE7 = {
   { _GPIO_FTM3_CH2_PTE7, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM3_PTD2 = {
   { _GPIO_FTM3_CH2_PTD2, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM3_PTE8 = {
   { _GPIO_FTM3_CH3_PTE8, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM3_PTD3 = {
   { _GPIO_FTM3_CH3_PTD3, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM3_PTE9 = {
   { _GPIO_FTM3_CH4_PTE9, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM3_PTC8 = {
   { _GPIO_FTM3_CH4_PTC8, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },

     _GPIO_Module_PWM3_PTE10 = {
   { _GPIO_FTM3_CH5_PTE10, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
     _GPIO_Module_PWM3_PTC9 = {
   { _GPIO_FTM3_CH5_PTC9, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM3_PTE11 = {
   { _GPIO_FTM3_CH6_PTE11, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
   _GPIO_Module_PWM3_PTC10 = {
   { _GPIO_FTM3_CH6_PTC10, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
     _GPIO_Module_PWM3_PTE12 = {
   { _GPIO_FTM3_CH7_PTE12, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
   
     _GPIO_Module_PWM3_PTC11 = {
   { _GPIO_FTM3_CH7_PTC11, -1, -1},
   { _PWM_CONFIG, -1, -1}
   },
  //// SPI ////
  _GPIO_Module_SPI0_PA15_17_16 = {
    {_GPIO_SPI0_SCK_PTA15, _GPIO_SPI0_MISO_PTA17, _GPIO_SPI0_MOSI_PTA16, -1},
    {_SPI_CONFIG, _SPI_CONFIG, _SPI_CONFIG, -1}
  },
  
  _GPIO_Module_SPI0_PC5_7_6 = {
    {_GPIO_SPI0_SCK_PTC5, _GPIO_SPI0_MISO_PTC7, _GPIO_SPI0_MOSI_PTC6, -1},
    {_SPI_CONFIG, _SPI_CONFIG, _SPI_CONFIG, -1}
  },
  
  _GPIO_Module_SPI0_PD1_3_2 = {
    {_GPIO_SPI0_SCK_PTD1, _GPIO_SPI0_MISO_PTD3, _GPIO_SPI0_MOSI_PTD2, -1},
    {_SPI_CONFIG, _SPI_CONFIG, _SPI_CONFIG, -1}
  },
  
  _GPIO_Module_SPI1_PD5_7_6 = {
    {_GPIO_SPI1_SCK_PTD5, _GPIO_SPI1_MISO_PTD7, _GPIO_SPI1_MOSI_PTD6, -1},
    {_SPI_CONFIG, _SPI_CONFIG, _SPI_CONFIG, -1}
  },
  
  _GPIO_Module_SPI2_PB21_23_22 = {
    {_GPIO_SPI2_SCK_PTB21, _GPIO_SPI2_MISO_PTB23, _GPIO_SPI2_MOSI_PTB22, -1},
    {_SPI_CONFIG, _SPI_CONFIG, _SPI_CONFIG, -1}
  },
  
  //// I2C ////
  _GPIO_Module_I2C0_PE24_25 = {
    {_GPIO_I2C0_SCL_PTE24, _GPIO_I2C0_SDA_PTE25, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },
  
  _GPIO_Module_I2C0_PB0_1 = {
    {_GPIO_I2C0_SCL_PTB0, _GPIO_I2C0_SDA_PTB1, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },
  
  _GPIO_Module_I2C0_PB2_3 = {
    {_GPIO_I2C0_SCL_PTB2, _GPIO_I2C0_SDA_PTB3, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },
  
  _GPIO_Module_I2C0_PD2_3 = {
    {_GPIO_I2C0_SCL_PTD2, _GPIO_I2C0_SDA_PTD3, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },
  
  _GPIO_Module_I2C0_PD8_9 = {
    {_GPIO_I2C0_SCL_PTD8, _GPIO_I2C0_SDA_PTD9, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },
  
  _GPIO_Module_I2C1_PE1_0 = {
    {_GPIO_I2C1_SCL_PTE1, _GPIO_I2C1_SDA_PTE0, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },
  
  _GPIO_Module_I2C1_PC10_11 = {
    {_GPIO_I2C1_SCL_PTC10, _GPIO_I2C1_SDA_PTC11, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },
  
  _GPIO_Module_I2C2_PA12_11 = {
    {_GPIO_I2C2_SCL_PTA12, _GPIO_I2C2_SDA_PTA11, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },
  
  _GPIO_Module_I2C2_PA14_13 = {
    {_GPIO_I2C2_SCL_PTA14, _GPIO_I2C2_SDA_PTA13, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },
  
  _GPIO_Module_I2C3_PE11_10 = {
    {_GPIO_I2C3_SCL_PTE11, _GPIO_I2C3_SDA_PTE10, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },

  _GPIO_Module_I2C3_PA2_1 = {
    {_GPIO_I2C3_SCL_PTA2, _GPIO_I2C3_SDA_PTA1, -1},
    {_I2C_CONFIG, _I2C_CONFIG, -1}
  },
  
  //// CAN ////
  _GPIO_Module_CAN0_PA13_12 = {
    {_GPIO_CAN0_RX_PTA13, _GPIO_CAN0_TX_PTA12, -1},
    {_CAN_CONFIG, _CAN_CONFIG, -1}
  },
  
  _GPIO_Module_CAN0_PB19_18 = {
    {_GPIO_CAN0_RX_PTB19, _GPIO_CAN0_TX_PTB18, -1},
    {_CAN_CONFIG, _CAN_CONFIG, -1}
  },

  _GPIO_Module_CAN1_PE24_25 = {
    {_GPIO_CAN1_RX_PTE25, _GPIO_CAN1_TX_PTE24, -1},
    {_CAN_CONFIG, _CAN_CONFIG, -1}
  },

  _GPIO_Module_CAN1_PC16_17 = {
    {_GPIO_CAN1_RX_PTC16, _GPIO_CAN1_TX_PTC17, -1},
    {_CAN_CONFIG, _CAN_CONFIG, -1}
  },
  
  //// SDHC ////
  _GPIO_Module_SDHC_D0_D7 = {
    {_GPIO_SDHC_D0_PTE1,   _GPIO_SDHC_D1_PTE0,  _GPIO_SDHC_D2_PTE5,     _GPIO_SDHC_D3_PTE4, 
     _GPIO_SDHC_D4_PTD12,  _GPIO_SDHC_D5_PTD13, _GPIO_SDHC_D6_PTD14,    _GPIO_SDHC_D7_PTD15,
     _GPIO_SDHC_DCLK_PTE2, _GPIO_SDHC_CMD_PTE3, -1},
    
    {_SDHC_CONFIG, _SDHC_CONFIG, _SDHC_CONFIG, _SDHC_CONFIG,
     _SDHC_CONFIG, _SDHC_CONFIG, _SDHC_CONFIG, _SDHC_CONFIG,
     _SDHC_CONFIG, _SDHC_CONFIG, -1}
  },
  
  _GPIO_Module_SDHC_D0_D3 = {
    {_GPIO_SDHC_D0_PTE1,   _GPIO_SDHC_D1_PTE0,  _GPIO_SDHC_D2_PTE5,     _GPIO_SDHC_D3_PTE4,
     _GPIO_SDHC_DCLK_PTE2, _GPIO_SDHC_CMD_PTE3, -1},
    
    {_SDHC_CONFIG, _SDHC_CONFIG, _SDHC_CONFIG, _SDHC_CONFIG,
     _SDHC_CONFIG, _SDHC_CONFIG, -1}
  },

  //// Ethernet ////
  
//// Ethernet ////
  _GPIO_Module_ETH = {
     {_GPIO_ETH_TXD1_PTA17, 
     _GPIO_ETH_TXD0_PTA16, 
     _GPIO_ETH_TXEN_PTA15,
     _GPIO_ETH_MDC_PTA8, 
     _GPIO_ETH_MDIO_PTA7, 
     _GPIO_ETH_CRS_DV_PTA14, 
     _GPIO_ETH_RXER_PTA5, 
     _GPIO_ETH_RXD0_PTA13,
     _GPIO_ETH_RXD1_PTA12, 
     _GPIO_ETH_CLKIN_PTE26, -1},

    {_ETH_CONFIG, _ETH_CONFIG, _ETH_CONFIG,
     _ETH_CONFIG, _ETH_CONFIG, _ETH_CONFIG, _ETH_CONFIG,
     _ETH_CONFIG, _ETH_CONFIG, _ETH_CONFIG, -1}
  };