/*
 * Project name:
     FLASH_KINETIS
 * Copyright:
     (c) Strahinja Jacimovic
 * Revision History:
     Initial test.
 * Status:
     Tested.
 * Description:
     Reads from JEDEC register (manufacturer ID,device type, device ID) and
     writes it on display.
     Writes to on-board FLASH module. Reads from it.
     Compares the data and writes results on display.
 * Test configuration:
     MCU:             MKV58F1M0XXX24
     Dev.Board:       Mikromedia 7' board
     Oscillator:      237.500000
     Ext. Modules:    None
     SW:              mikroC PRO for ARM v6.1.0
 * NOTES:
     - NONE
 */

#include "Flash_driver.h"
#define ARRAY_LENGTH 150

// Global connections
sbit Chip_Select at GPIOB_PDOR.B17;
// End of global connections

// Global variables
unsigned char counter = 0;
char someData[ARRAY_LENGTH];
char rData[ARRAY_LENGTH];
// End Global variables

uint8_t tmp[3],
        man_id[3],
        dev_type[3],
        dev_id[3];
        
/*******************************************************************************
* Procedure Init_MCU(void)
* ------------------------------------------------------------------------------
* Overview: Function initializes required modules for example
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Init_MCU(void) {

  GPIO_Digital_Output(&GPIOB_PDOR, _GPIO_PINMASK_17);
  Chip_Select = 1;                             // Deselect SerialRAM

  SPI0_Init_Advanced(1000000, _SPI_CFG_MASTER |
                              _SPI_CFG_FRAME_SIZE_8BITS |
                              _SPI_CFG_CLK_IDLE_LOW |
                              _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                              _SPI_CFG_MSB_FIRST, &_GPIO_Module_SPI0_PC5_7_6 );
  Delay_100ms();
  
  Flash_Reset();
  Delay_100ms();
  Flash_Global_Unlock();
  Flash_Chip_Erase();

  memset(someData, 0x00, sizeof(someData));
  memset(rData, 0x01, sizeof(someData));       // something that's not zero
  
}
/*******************************************************************************
* Procedure JEDEC_READ(void)
* ------------------------------------------------------------------------------
* Overview: Function initializes required modules for example
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void JEDEC_READ(void) {

   Read_Jedec_ID(tmp);

   ByteToHex(tmp[0], man_id);
   ByteToHex(tmp[1], dev_type);
   ByteToHex(tmp[2], dev_id);

   TFT_Write_Text ("JEDEC-ID:", 360, 260);
   Delay_100ms();
   TFT_Write_Text (man_id, 360, 280);
   TFT_Write_Text (dev_type, 380, 280);
   TFT_Write_Text (dev_id, 400, 280);

}
/*******************************************************************************
* Procedure data_written(void)
* ------------------------------------------------------------------------------
* Overview: Function calls Write_Data and writes it via UART
* Input: Nothing
* Output: Data written
*******************************************************************************/
void data_written(void) {
  TFT_Write_Text("Writing array to on-board FLASH...", 280, 180);
  for (counter = 0; counter < sizeof(someData); counter++) {
    Flash_Page_Program(0x006400 + counter, counter);           // Write data into SRAM on 0x006400 address
    someData[counter] = counter;
    Delay_ms(5);
  }
}
/*******************************************************************************
* Procedure data_read(void)
* ------------------------------------------------------------------------------
* Overview: Function calls Read_Data and writes via UART
* Input: Nothing
* Output: Data read
*******************************************************************************/
void data_read(void) {
int counter = 0;
  TFT_Write_Text("Reading array from on-board FLASH...", 280, 210);
  for (counter = 0; counter< sizeof(someData); counter++) {
    rData[counter] = Flash_Read_40MHz(0x006400 + counter);    // Read data from SRAM from 0x006400 address                                                               // Delay of 25 miliseconds
    Delay_ms(5);
  }
  Delay_1sec();
  TFT_Fill_Screen(CL_BLACK);

  if (!memcmp(someData , rData, sizeof(someData)))
    TFT_Write_Text("Read data is the same as written. Everything is OK!!!", 240, 200);
  else {
    TFT_Write_Text("Read data is NOT the same as written. Something went wrong!!!", 230, 200);
    while (1);
  }
  
  Delay_ms(3000);

}
//******************************************************************************
// Main procedure
//******************************************************************************
void main() {

   Init_MCU();                                // Initialize peripherals
   init_display();                            // Initialize display
   
   data_written();
   data_read();
   
   JEDEC_READ();
   
   while(1);

}
////////////////////////////////////////////////////////////////////////////////