#include "Flash_driver.h"

// Global connections
sbit Chip_Select_Intern at GPIOB_PDOR.B17;
// End of global connections

/*******************************************************************************
* Check if FLASH is busy
*******************************************************************************/
uint8_t Is_Flash_Busy(void) {
uint8_t temp;

   Chip_Select_Intern = 0;
   SPI_Write(RDSR);
   Delay_1us();
   temp = SPI_Read(Dummy_Data);
   Chip_Select_Intern = 1;
   return temp;

}
/*******************************************************************************
* Enable SPI write
*******************************************************************************/
void Flash_Write_Enable(void) {

   Chip_Select_Intern = 0;
   SPI_Write(WREN);
   Chip_Select_Intern = 1;

}
/*******************************************************************************
* Clear all write protection bits
*******************************************************************************/
void Flash_Global_Unlock(void) {

   Flash_Write_Enable();
   
   Chip_Select_Intern = 0;
   SPI_Write(ULBPR);
   Chip_Select_Intern = 1;

}
/*******************************************************************************
* Perform a software RESET
*******************************************************************************/
void Flash_Reset(void) {

   Chip_Select_Intern = 0;
   SPI_Write(RSTEN);
   Chip_Select_Intern = 1;
   Delay_1us();
   Chip_Select_Intern = 0;
   SPI_Write(RST);
   Chip_Select_Intern = 1;

}
/*******************************************************************************
* Read from FLASH at 40MHz SPI speed max
*******************************************************************************/
uint8_t Flash_Read_40MHz(uint32_t address) {
uint8_t temp;

   Chip_Select_Intern = 0;
   SPI_Write(READ);
   Delay_1us();
   SPI_Write((uint8_t)(address >> 16));
   SPI_Write((uint8_t)((address & 0x00FF00) >> 8));
   SPI_Write((uint8_t)(address & 0x0000FF));
   Delay_1us();
   temp = SPI_Read(Dummy_Data);
   Chip_Select_Intern = 1;
   
   while (Is_Flash_Busy());
   return temp;
   
}
/*******************************************************************************
* Write to FLASH at desired address
*******************************************************************************/
void Flash_Page_Program(uint32_t address, uint8_t dat) {

   Flash_Write_Enable();

   Chip_Select_Intern = 0;
   SPI_Write(PP);
   Delay_1us();
   SPI_Write((uint8_t)(address >> 16));
   SPI_Write((uint8_t)((address & 0x00FF00) >> 8));
   SPI_Write((uint8_t)(address & 0x0000FF));
   Delay_1us();
   SPI_Write(dat);
   Chip_Select_Intern = 1;
   
   while (Is_Flash_Busy());

}
/*******************************************************************************
* Enable Quad SPI mode
*******************************************************************************/
void Flash_Enable_Quad(void) {

   Chip_Select_Intern = 0;
   SPI_Write(EQIO);
   Chip_Select_Intern = 1;
   
   while (Is_Flash_Busy());

}
/*******************************************************************************
* Reset QUAD SPI mode
*******************************************************************************/
void Flash_Reset_Quad(void) {

   Chip_Select_Intern = 0;
   SPI_Write(RSTQIO);
   Chip_Select_Intern = 1;
   
   while (Is_Flash_Busy());

}
/*******************************************************************************
* Erase all memory locations
*******************************************************************************/
void Flash_Chip_Erase(void) {

   Flash_Write_Enable();
   
   Chip_Select_Intern = 0;
   SPI_Write(CE);
   Chip_Select_Intern = 1;
   
   while (Is_Flash_Busy());

}
/*******************************************************************************
* Read ID register
*******************************************************************************/
void Read_Jedec_ID(uint8_t jedec_id_string[3]) {
uint8_t i;
   
   Chip_Select_Intern = 0;
   SPI_Write(JEDECID);

   for (i = 0; i < 3; i++)
      jedec_id_string[i] = SPI_Read(Dummy_Data);

   Chip_Select_Intern = 1;

   while (Is_Flash_Busy());
   
}
////////////////////////////////////////////////////////////////////////////////