// GLOBAL DEFINES
/*

  LEGEND:
  (1) RST command only executed if RSTEN command is executed first.
      Any intervening command will disable Reset.
  (2) Device accepts eight-clock command in SPI mode, or two-clock command in SQI mode.
  (3) Data cycles are two clock periods. 
      IOC bit must be set to 1 before issuing the command.
  (4) Address, Dummy/Mode bits, and data cycles are two clock periods.
      IOC bit must be set to 1 before issuing the command.
  (5) Data cycles are four clock periods.
  (6) Address, Dummy/Mode bits, and Data cycles are four clock periods.
  (7) Sector Addresses: Use AMS - A12, remaining address are dont care,
      but must be set to VIL or VIH.
  (8) Blocks are 64 KByte, 32 KByte, or 8KByte, depending on location.
      Block Erase Address: AMS - A16 for 64 KByte; AMS - A15
      for 32 KByte; AMS - A13 for 8 KByte. Remaining addresses are dont care,
      but must be set to VIL or VIH.

*/
#include "stdint.h"

// Configuration
#define  NoOP       0x00     // No operation
#define  RSTEN      0x66     // Reset enable
#define  RST        0x99     // Reset memory   (1)
#define  EQIO       0x38     // Enable Quad I/O
#define  RSTQIO     0xFF     // Reset Quad I/O    (2)
#define  RDSR       0x05     // Read status register
#define  WRSR       0x01     // Write status register
#define  RDCR       0x35     // Read configuration register
// Read
#define  READ       0x03     // Read memory
#define  HREAD      0x0B     // Read memory at higher speed
#define  SQOR       0x6B     // SPI Quad Output Read   (3)
#define  SQIOR      0xEB     // SPI Quad I/O Read   (4)
#define  SDOR       0x3B     // SPI Dual Output Read   (5)
#define  SDIOR      0xBB     // SPI Dual I/O Read   (6)
#define  SB         0xC0     // Set Burst Length
#define  RBSQI      0x0C     // SQI Read Burst with Wrap
#define  RBSPI      0xEC     // SPI Read Burst with Wrap   (4)
#define  Dummy_Data 0x00    // Dummy data for reading
// Identification
#define  JEDECID    0x9F     // JEDEC-ID Read
#define  QUADJID    0xAF     // Quad I/O J-ID Read
#define  SFDP       0x5A     // Serial Flash Discoverable Parameters
// Write
#define  WREN       0x06     // Write Enable
#define  WRDI       0x04     // Write Disable
#define  SE         0x20     // Erase 4 KBytes of Memory Array   (7)
#define  BE         0xD8     // Erase 64, 32 or 8 KBytes of Memory Array   (8)
#define  CE         0xC7     // Erase Full Array
#define  PP         0x02     // Page Program
#define  SPIQUADPP  0x32     // SQI Quad Page Program   (3)
#define  WRSU       0xB0     // Suspends Program/Erase
#define  WRRE       0x30     // Resumes Program/Erase
// Protection
#define  RBPR       0x72     // Read Block-Protection Register
#define  WBPR       0x42     // Write Block-Protection Register
#define  LBPR       0x8D     // Lock Down Block-Protection Register
#define  NVWLDR     0xE8     // non-Volatile Write LockDown Register
#define  ULBPR      0x98     // Global Block Protection Unlock
#define  RSID       0x88     // Read Security ID
#define  PSID       0xA5     // Program User Security ID area
#define  LSID       0x85     // Lockout Security ID Programming

void Flash_Reset(void);
uint8_t Flash_Read_40MHz(uint32_t address);
void Flash_Enable_Quad(void);
void Flash_Reset_Quad(void);
void Read_Jedec_ID(uint8_t jedec_id_string[3]);
void Flash_Page_Program(uint32_t address, uint8_t dat);
void Flash_Write_Enable(void);
void Flash_Global_Unlock(void);
void Flash_Chip_Erase(void);
void init_display(void);