#include "built_in.h"
#include "__Lib_Mmc.h"
// TFT module connections
sbit TFT_CS at GPIOB_PDOR.B20;
char TFT_DataPort at GPIOD_PDOR;
sbit TFT_RD at GPIOB_PDOR.B22;
sbit TFT_RS at GPIOB_PDOR.B21;
sbit TFT_RST at GPIOB_PDOR.B19;
sbit TFT_WR at GPIOB_PDOR.B23;
// End TFT module connections

// MMC Connections
sbit Mmc_Chip_Select           at GPIOB_PDOR.B8;
sbit MMC_Card_Detect           at GPIOB_PDIR.B9;

// MP3 Codec Connections
sbit MP3_CS                    at GPIOA_PDOR.B6;
sbit MP3_RST                   at GPIOA_PDOR.B10;
sbit BSYNC                     at GPIOA_PDOR.B24;
sbit DREQ1                     at GPIOA_PDIR.B11;

// VS1053E constants
const char WRITE_CODE           = 0x02;
const char READ_CODE            = 0x03;

const char SCI_BASE_ADDR        = 0x00;
const char SCI_MODE_ADDR        = 0x00;
const char SCI_STATUS_ADDR      = 0x01;
const char SCI_BASS_ADDR        = 0x02;
const char SCI_CLOCKF_ADDR      = 0x03;
const char SCI_DECODE_TIME_ADDR = 0x04;
const char SCI_AUDATA_ADDR      = 0x05;
const char SCI_WRAM_ADDR        = 0x06;
const char SCI_WRAMADDR_ADDR    = 0x07;
const char SCI_HDAT0_ADDR       = 0x08;
const char SCI_HDAT1_ADDR       = 0x09;
const char SCI_AIADDR_ADDR      = 0x0A;
const char SCI_VOL_ADDR         = 0x0B;
const char SCI_AICTRL0_ADDR     = 0x0C;
const char SCI_AICTRL1_ADDR     = 0x0D;
const char SCI_AICTRL2_ADDR     = 0x0E;
const char SCI_AICTRL3_ADDR     = 0x0F;

const BYTES_2_WRITE = 32;
const BUFFER_SIZE = 256;

char MP3_Example_State = 0;
char Fat_Initialized_Flag;
const unsigned long mp3_note_bmp;
const unsigned long mp3_player_note_bmp;
char level=100, old_level=0;
signed short Ext_fhandle;
char Ext_res_initialized;

unsigned long Buffer_count, Total_Buffers;
char MP3_song[13];
char current_fhandle;
signed short current_file;
int MP3_Example_cnt = 0;
unsigned long file_size;
char mp3_buffer[BUFFER_SIZE];
char volume_left, volume_right;

//sd card variables
char Fat_Init_Flag;
short File_Exist_Flag;
short File_Handle = 0xFF;

char buff[6];

unsigned int* buffer;
char temp1;

void Init_Display() {

  TFT_Init_SSD1963_Board_50(800, 480);

  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
}

//--------------- SD CARD ---------------
/*void Init_SDIO(){
    // Initialize SDIO
    SDIO_Reset();

    SDIO_Init(_SDIO_CFG_POWER_SAVE_DISABLE | _SDIO_CFG_4_WIDE_BUS_MODE | _SDIO_CFG_CLOCK_BYPASS_DISABLE
            | _SDIO_CFG_CLOCK_RISING_EDGE | _SDIO_CFG_HW_FLOW_DISABLE, 125, &_GPIO_MODULE_SDIO_D0_D3);

    // Set pull-ups on SDIO lines
    GPIOD_PUPDRbits.PUPDR2 = 1;
    GPIOC_PUPDRbits.PUPDR8 = 1;
    GPIOC_PUPDRbits.PUPDR9 = 1;
    GPIOC_PUPDRbits.PUPDR10 = 1;
    GPIOC_PUPDRbits.PUPDR11 = 1;

    Mmc_Set_Interface(_MMC_INTERFACE_SDIO);
}*/

void Init_FAT(){
  char FAT_cnt = 0;
  Fat_Init_Flag = 0;
  if (MMC_Card_Detect == 0){
    if (Fat_Init_Flag == 0){
      while ((FAT32_Init() != 0) && (FAT_cnt < 5))
        FAT_cnt ++;
      if (FAT_cnt < 5){
            SPI1_Init_Advanced(1000000,   _SPI_CFG_MASTER |
                                _SPI_CFG_FRAME_SIZE_8BITS |
                                _SPI_CFG_CLK_IDLE_LOW |
                                _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                                _SPI_CFG_MSB_FIRST, &_GPIO_Module_SPI1_PE2_3_1 );
           Fat_Init_Flag = 1;
      }
    }
  }
}
//---------------------------------------------------------------------

//--------------- MP3 ---------------

void MP3_SCI_Write(char address, unsigned int data_in) {
  BSYNC = 1;

  MP3_CS = 0;                    // select MP3 SCI
  SPI0_Write(WRITE_CODE);
  SPI0_Write(address);
  SPI0_Write(Hi(data_in));       // high byte
  SPI0_Write(Lo(data_in));       // low byte
  MP3_CS = 1;                    // deselect MP3 SCI
  while (DREQ1 == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

void MP3_SCI_Read(char start_address, char words_count, unsigned int *data_buffer) {
  unsigned int temp;

  MP3_CS = 0;                    // select MP3 SCI
  SPI0_Write(READ_CODE);
  SPI0_Write(start_address);

  while (words_count--) {        // read words_count words byte per byte
    temp = SPI0_Read(0);
    temp <<= 8;
    temp += SPI0_Read(0);
    *(data_buffer++) = temp;
  }
  MP3_CS = 1;                    // deselect MP3 SCI
  while (DREQ1 == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI
}

void MP3_SDI_Write(char data_) {

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ1 == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  SPI0_Write(data_);
  BSYNC = 1;
}

void MP3_Reset_Time(){
  MP3_SCI_Write(SCI_DECODE_TIME_ADDR, 0);
}

void MP3_SDI_Write_32(char *data_) {
  char i;

  MP3_CS = 1;
  BSYNC = 0;

  while (DREQ1 == 0);             // wait until DREQ becomes 1, see MP3 codec datasheet, Serial Protocol for SCI

  for (i=0; i<32; i++)
  SPI0_Write(data_[i]);
  BSYNC = 1;
}

void MP3_Set_Volume(char left, char right) {
  unsigned int volume;

  volume = (left<<8) + right;             // calculate value
  MP3_SCI_Write(SCI_VOL_ADDR, volume);    // Write value to VOL register
}

void MP3_Set_default_Mode(){
  BSYNC = 1;
  MP3_CS = 1;

  // Hardware reset
  MP3_RST = 0;
  Delay_ms(100);
  MP3_RST = 1;
  while (DREQ1 == 0);
  MP3_SCI_Write(SCI_MODE_ADDR, 0x0800);
  MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
  MP3_SCI_Write(SCI_CLOCKF_ADDR, 0xC000);   // default 12 288 000 Hz
  // Maximum volume is 0x00 and total silence is 0xFE.
}

void MP3_Start() {

  GPIO_Digital_Input (&GPIOA_PDOR, _GPIO_PINMASK_11);
  GPIO_Digital_Output(&GPIOA_PDOR, _GPIO_PINMASK_6);
  GPIO_Digital_Output(&GPIOA_PDOR, _GPIO_PINMASK_10);
  GPIO_Digital_Output(&GPIOA_PDOR, _GPIO_PINMASK_24);

  MP3_CS            = 1;               // Deselect MP3_CS
  MP3_RST           = 1;               // Set MP3_RST pin
  BSYNC             = 0;               // Clear BSYNC
  BSYNC             = 1;               // Clear BSYNC

  // Initialize SPI0 module
  SPI0_Init_Advanced(1000000,   _SPI_CFG_MASTER |
                                _SPI_CFG_FRAME_SIZE_8BITS |
                                _SPI_CFG_CLK_IDLE_LOW |
                                _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                                _SPI_CFG_MSB_FIRST, &_GPIO_Module_SPI0_PC5_7_6 );

  MP3_Set_default_Mode();

  // Initialize SPI0 module at higher speed
  SPI0_Init_Advanced(1000000,   _SPI_CFG_MASTER |
                                _SPI_CFG_FRAME_SIZE_8BITS |
                                _SPI_CFG_CLK_IDLE_LOW |
                                _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                                _SPI_CFG_MSB_FIRST, &_GPIO_Module_SPI0_PC5_7_6 );

  MP3_Set_Volume(100 - level, 100 - level);
  MP3_Reset_Time();
}

void Mp3_Init(){

  //Init_FAT();
  if (MMC_Card_Detect == 1){
    MP3_Example_State = 0;
    return;
  }
  MP3_Example_State = 1;
  MP3_Start();
}

void doMP3(){
 unsigned long i;
  switch (MP3_Example_State){
    case 0 : { // mp3 example started
               if (MMC_Card_Detect == 0){        // if SD card detected
                 Delay_ms(100);
                 Mp3_Init();
                 MP3_Example_State = 1;
               }
                 MP3_Example_cnt++;
                 if (MP3_Example_cnt > 599)
                   MP3_Example_cnt = 0;
               //}
             }; break;
    case 1 : { // stop playing
             }; break;                           // if mp3 is stopped, do nothing
    case 2 : { // play song
               if (file_size > BUFFER_SIZE){     // while there is still data, send it to mp3 codec chip
                 SPI_Set_Active(SPI1_Read, SPI1_Write);
                 FAT32_Read(current_fhandle, mp3_buffer, BUFFER_SIZE);
                 SPI_Set_Active(SPI2_Read, SPI2_Write);
                 for (i=0; i<BUFFER_SIZE/BYTES_2_WRITE; i++) {
                   MP3_SDI_Write_32(mp3_buffer + i*BYTES_2_WRITE);
                 }
                 file_size -= BUFFER_SIZE;
                 Buffer_count ++;
               }
               if (file_size < BUFFER_SIZE){     // finish playing file
                 SPI_Set_Active(SPI1_Read, SPI1_Write);
                 FAT32_Read(current_fhandle, mp3_buffer, file_size);
                 SPI_Set_Active(SPI2_Read, SPI2_Write);
                 for (i=0; i<file_size; i++){
                   MP3_SDI_Write(mp3_buffer[i]);
                 }
                 MP3_Example_State = 1;
                 Buffer_count ++;
               }

             }; break;
    case 3 : { // pause playing
             }; break;
  }
}

void MP3_play(char *Song_Name){
  Buffer_count = 0;
  current_fhandle = FAT32_Open("sound.mp3", 0x01);
  FAT32_Size("sound.mp3", &file_size);
  Buffer_count = 0;
  Total_Buffers = file_size / BUFFER_SIZE;
  SPI_Set_Active(SPI2_Read, SPI2_Write);
  MP3_Reset_Time();
  MP3_Set_Volume(100 - level, 100 - level);
  MP3_Example_State = 2;
}

//---------------------------------------------------------------------
Mmc_Timeout_Values timeoutMMC;

// declare timeout callback function
void Mmc_TimeoutCallback(char errorCode) {
  // if there was error during the INIT sequence
  if (errorCode == _MMC_INIT_TIMEOUT) {

  }

  // if there was error during the CMD sequence
  if (errorCode == _MMC_CMD_TIMEOUT) {

  }

  // if there was error during the SPI sequence
  if (errorCode == _MMC_SPI_TIMEOUT) {

  }
}


void main() {
//  SD CARD
//  GPIO_Digital_Input(&GPIOD_BASE, _GPIO_PINMASK_3);   //SD_CD
//  GPIO_Digital_Output(&GPIOD_BASE, _GPIO_PINMASK_2);   //SD_CMD
  GPIO_Digital_Output(&GPIOD_PDOR, _GPIO_PINMASK_ALL);      //TFT data port

  // initialize timeout structure
  timeoutMMC.cmd_timeout  = 100000;
  timeoutMMC.spi_timeout  = 100000;
  timeoutMMC.init_timeout = 100000;
  Mmc_SetTimeoutCallback(&timeoutMMC, &Mmc_TimeoutCallback);
  Delay_ms(200);
  
  Init_Display();

  TFT_Fill_Screen(CL_BLUE);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Set_Pen(CL_WHITE,1);
  TFT_Rectangle(10,10,310,230);
  SPI1_Init_Advanced(1000000,   _SPI_CFG_MASTER |
                                _SPI_CFG_FRAME_SIZE_8BITS |
                                _SPI_CFG_CLK_IDLE_LOW |
                                _SPI_CFG_CLK_PHASE_CAPT_LEADING |
                                _SPI_CFG_MSB_FIRST, &_GPIO_Module_SPI1_PE2_3_1 );
  TFT_Write_Text("Initialize SD Card...", 20,20);
  //Init_SDIO();
  Init_FAT();
  if(Fat_Init_Flag)
  {
    TFT_Write_Text("Initialization success", 20,40);
  }
  else
  {
    TFT_Write_Text("Initialization failed", 20,40);
    while(1);
  }

  Mp3_Init();
  SPI_Set_Active(SPI1_Read, SPI1_Write);
  current_fhandle = FAT32_Open("sound.mp3", 0x01);
  MP3_play("sound.mp3");
  TFT_Write_Text("Play sound.mp3", 20,60);
  while (1)
  {
    doMP3();
  }
}