#include <stdint.h>

// Slave address
#define SLAVE_ADDRESS         ( 0x29 )

// ALS operation mode control SW reset
#define ALS_CONTR_REG         ( 0x80 )

// ALS measurement rate in active mode
#define ALS_MEAS_REG          ( 0x85 )

// Part number ID and revision ID
#define PART_ID_REG           ( 0x86 )

// Manufacturer ID
#define MANUFAC_ID_REG        ( 0x87 )

// ALS measurement CH1 data, lower byte
#define ALS_DATA_CH1_REG0     ( 0x88 )

// ALS measurement CH1 data, upper byte
#define ALS_DATA_CH1_REG1     ( 0x89 )

// ALS measurement CH0 data, lower byte
#define ALS_DATA_CH0_REG0     ( 0x8A )

// ALS measurement CH0 data, upper byte
#define ALS_DATA_CH0_REG1     ( 0x8B )

// ALS data status
#define ALS_STATUS_REG        ( 0x8C )

void OPT_WrSingle(unsigned short wAddr, unsigned short wData);
unsigned short OPT_RdSingle(unsigned short rAddr);
void OPT_RdSeq(unsigned short rAddr,
                        unsigned char *rdData,
                        unsigned long rLen);

uint8_t data_[256];
uint8_t rData[28];
uint8_t txt01[4];
uint8_t txt11[6];
uint8_t txt2[6];
uint8_t txt3[4];
uint16_t sensorData[10];
uint8_t part_id_reg;
uint8_t manufac_id_reg;

// TFT module connections
sbit TFT_CS at GPIOB_PDOR.B20;
char TFT_DataPort at GPIOD_PDOR;
sbit TFT_RD at GPIOB_PDOR.B22;
sbit TFT_RS at GPIOB_PDOR.B21;
sbit TFT_RST at GPIOB_PDOR.B19;
sbit TFT_WR at GPIOB_PDOR.B23;

void Init_Display() {
  TFT_Init_SSD1963_Board_50(800, 480);
  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
}

void OPT_Init() {
  I2C0_Init_Advanced(100000, &_GPIO_MODULE_I2C0_PB2_3);
  OPT_WrSingle(0x80, 0x01);               // Active mode
  OPT_WrSingle(0x85, 0x12);               // Active mode
}//~

//--------------- Writes data to FXOS8700CQ - signle location
void OPT_WrSingle(unsigned short wAddr, unsigned short wData) {
  data_[0] = wAddr;
  data_[1] = wData;
  I2C0_Start();
  // issue I2C start signal
  I2C0_Write(SLAVE_ADDRESS, data_, 2, END_MODE_STOP);
}//~

//--------------- Reads data from FXOS8700CQ - single location (random)
unsigned short OPT_RdSingle(unsigned short rAddr) {
  data_[0] = rAddr;
  I2C0_Start();              // issue I2C start signal
  I2C0_Write(SLAVE_ADDRESS, data_, 1, END_MODE_RESTART);
  I2C0_Read(SLAVE_ADDRESS, data_, 1, END_MODE_STOP);

  return data_[0];
}//~

//--------------- Reads data from FXOS8700CQ - sequential read

void OPT_RdSeq(unsigned short rAddr,
                        unsigned char *rdData,
                        unsigned long rLen) {
  data_[0] = rAddr;
  I2C0_Start();              // issue I2C start signal
  I2C0_Write(SLAVE_ADDRESS,data_,1,END_MODE_RESTART);
  I2C0_Read(SLAVE_ADDRESS, rdData, rLen, END_MODE_STOP);
}//~

void main() {
   Init_Display();
   OPT_Init();
   Delay_ms(100);
   TFT_Write_Text("PART ID = ", 65, 40);
   TFT_Write_Text("MANUFAC ID = ", 65, 70);
   TFT_Write_Text("CH0 = ", 65, 100);
   TFT_Write_Text("CH1 = ", 65, 130);
   part_id_reg = OPT_RdSingle(PART_ID_REG);
   ByteToHex(part_id_reg, txt01);
   manufac_id_reg = OPT_RdSingle(MANUFAC_ID_REG );
   ByteToHex(manufac_id_reg, txt3);
   TFT_Write_Text(txt01, 130, 40);
   TFT_Write_Text(txt3, 160, 70);
   while (1) {
       OPT_RdSeq(ALS_DATA_CH1_REG0, rData, 4);
       Delay_ms(100);
       sensorData[0] = ( rData[1] << 8 | rData[0] );
       sensorData[1] = ( rData[3] << 8 | rData[2] );
       WordToStr(sensorData[0], txt11);
       WordToStr(sensorData[1], txt2);
       Delay_ms(100);
       TFT_Set_Brush(1,CL_WHITE,0,0,0,0);
       TFT_Set_Pen(CL_WHITE,1);
       TFT_Rectangle(110,100,165,160);
       TFT_Set_Font(TFT_defaultFont, CL_RED, FO_HORIZONTAL);
       TFT_Write_Text(txt11, 110, 100);
       TFT_Write_Text("lux", 170, 100);
       TFT_Write_Text(txt2, 110, 130);
       TFT_Write_Text("lux", 170, 130);
   }
}