/*******************************************************************************
* RTC Driver - Function Set used for RTC module
* ------------------------------------------------------------------------------
*
*******************************************************************************/
// TFT module connections
sbit TFT_BLED at GPIOA_PDOR.B4;
sbit TFT_CS at GPIOB_PDOR.B20;
char TFT_DataPort at GPIOD_PDOR;
sbit TFT_RD at GPIOB_PDOR.B22;
sbit TFT_RS at GPIOB_PDOR.B21;
sbit TFT_RST at GPIOB_PDOR.B19;
sbit TFT_WR at GPIOB_PDOR.B23;
// End TFT module connections
void Init_Display() {
  TFT_Init_SSD1963_Board_50(800, 480);
  TFT_Set_DBC_SSD1963(255);
  TFT_Fill_Screen(CL_WHITE);
}
// Time struct with am/pm
typedef struct MyTime {
  short ampm;
  short seconds;
  short minutes;
  short hours;
  short day;
  short month;
  int year;
} TTime;

TTime MyTime = {0, 0, 0, 0, 0, 0};
char txt2[50];
unsigned long temp;
// variables
unsigned long RTC_Time, old_RTC_Time;

/*******************************************************************************
* Function RTC_Init()
* ------------------------------------------------------------------------------
* Overview: Function initializes RTC module
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void RTC_Init(){
  TFT_Write_Text("   SETTING RTC ACCESS  ", 550, 200);
  SIM_SCGC6 = 0x20000000;
  Delay_ms(200);
  TFT_Write_Text("   ENABLING RTC OSCILLATOR  ", 550, 220);
  RTC_CR = 0x100;
  Delay_ms(1000);
  TFT_Write_Text("   WRITE SECONDS TO TSR   ", 550, 240);
  if(!RTC_TSR)
  {
   RTC_SR.TCE = 0;
   Delay_ms(200);
   RTC_TSR = 0x0;
  }
  TFT_Write_Text("   ENABLING TIMER COUNTER  ", 550, 260);
  RTC_SR.TCE = 1;
  Delay_ms(200);
  TFT_Write_Text("   COUNTING STARTS  ", 550, 280);
}


/*******************************************************************************
* Function RTCC_Read(TTime *RTCC_Time)
* ------------------------------------------------------------------------------
* Overview: Function reads RTC Time and calendar
* Input: Time struct
* Output: Nothing
*******************************************************************************/
char RTCC_Read(TTime *RTCC_Time){
  char temp = 0;

  RTC_Time = RTC_TSR;

  if(RTC_Time != old_RTC_Time){
    old_RTC_Time = RTC_Time;
    RTCC_Time->seconds = Bcd2Dec((short)(RTC_Time & 0xffffff));
    temp = 1;
  }
  return temp;
}

void main() {
    Init_Display();
    RTC_Init();
    TFT_Write_Text("Seconds = ", 65, 100);
/*TFT_Write_Text("Minutes = ", 65, 130);
    TFT_Write_Text("  Hours = ", 65, 160);
    TFT_Write_Text("   AMPM = ", 65, 190);
    TFT_Write_Text("    Day = ", 65, 220);
    TFT_Write_Text("  Month = ", 65, 250);
    TFT_Write_Text("   Year = ", 65, 280);*/
    while(1)
    {
      TFT_Set_Pen(CL_WHITE,1);
      TFT_Rectangle(135,100,165,310);
      TFT_Set_Font(TFT_defaultFont, CL_RED, FO_HORIZONTAL);
      RTCC_Read(&MyTime);
      ShortToStr(MyTime.seconds, txt2);
      TFT_Write_Text(txt2, 135, 100);
/*ShortToStr(MyTime.minutes, txt2);
      TFT_Write_Text(txt1, 135, 130);
      ShortToStr(MyTime.hours, txt2);
      TFT_Write_Text(txt1, 135, 160);
      ShortToStr(MyTime.ampm, txt2);
      TFT_Write_Text(txt1, 135, 190);
      ShortToStr(MyTime.day, txt2);
      TFT_Write_Text(txt1, 135, 220);
      ShortToStr(MyTime.month, txt2);
      TFT_Write_Text(txt1, 135, 250);
      ShortToStr(MyTime.year, txt2);
      TFT_Write_Text(txt1, 135, 280);*/
      TFT_Set_Brush(1,CL_WHITE,0,0,0,0);

      Delay_ms(50);
    }
}