#include "TFT_43_objects.h"
#include "TFT_43_resources.h"
#include "built_in.h"


// TFT module connections
sbit TFT_BLED at GPIOA_PDOR.B4;
sbit TFT_CS at GPIOB_PDOR.B20;
char TFT_DataPort at GPIOD_PDOR;
sbit TFT_RD at GPIOB_PDOR.B22;
sbit TFT_RS at GPIOB_PDOR.B21;
sbit TFT_RST at GPIOB_PDOR.B19;
sbit TFT_WR at GPIOB_PDOR.B23;
// End TFT module connections

// Touch Panel module connections
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TButton *local_button;
TButton *exec_button;
int button_order;
TLabel *local_label;
TLabel *exec_label;
int label_order;
TBox *local_box;
TBox *exec_box;
int box_order;


void Write_to_Data_Lines(unsigned char _hi, unsigned char _lo) {
  GPIOD_PDOR = (unsigned int) (_lo | (_hi << 8));
}

void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Write_to_Data_Lines(0, index);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Write_to_Data_Lines(0, cmd);
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Write_to_Data_Lines(Hi(_data), Lo(_data));
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

static void InitializeTouchPanel() {
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init_SSD1963_Board_50(800, 480);

  TFT_Set_DBC_SSD1963(255);

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}

void Calibrate() {
  TFT_Fill_Screen(CL_BLACK);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Write_Text("Touch points on the screen as they", 125, 121);
  TFT_Write_Text("appear to calibrate touchscreen.", 135, 151);
  TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
  TFT_Circle(0, 0, 3);
  Delay_ms(500);
  TFT_Set_Brush(1, CL_BLACK, 0, 0, 0, 0);
  TFT_Circle(0, 0, 3);
  TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
  Delay_ms(500);
  TFT_Set_Brush(1, CL_BLACK, 0, 0, 0, 0);
  TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
  TFT_Circle(TFT_DISP_WIDTH-1, TFT_DISP_HEIGHT-1, 3);
  Delay_ms(500);
  TFT_Set_Brush(1, CL_BLACK, 0, 0, 0, 0);
  TFT_Circle(TFT_DISP_WIDTH-1, TFT_DISP_HEIGHT-1, 3);
  TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
  TFT_Set_Brush(1, CL_BLACK, 0, 0, 0, 0);
  Delay_ms(500);
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TLabel                 Label1;
char Label1_Caption[8] = "Screen1";

  TButton               Button1;
char Button1_Caption[8] = "Screen4";

  TButton               Button2;
char Button2_Caption[8] = "Screen2";

  TBox                   Box1;
  TBox                   Box2;
  TBox                   Box3;
  TBox                   Box4;
  TButton                * const code Screen1_Buttons[2]=
         {
         &Button1,             
         &Button2              
         };
  TLabel                 * const code Screen1_Labels[1]=
         {
         &Label1               
         };
  TBox                   * const code Screen1_Boxes[4]=
         {
         &Box1,                
         &Box2,                
         &Box3,                
         &Box4                 
         };

  TScreen                Screen2;
  TLabel                 Label2;
char Label2_Caption[8] = "Screen2";

  TButton               Button3;
char Button3_Caption[8] = "Screen1";

  TButton               Button4;
char Button4_Caption[8] = "Screen3";

  TBox                   Box9;
  TBox                   Box10;
  TBox                   Box11;
  TBox                   Box12;
  TButton                * const code Screen2_Buttons[2]=
         {
         &Button3,             
         &Button4              
         };
  TLabel                 * const code Screen2_Labels[1]=
         {
         &Label2               
         };
  TBox                   * const code Screen2_Boxes[4]=
         {
         &Box9,                
         &Box10,               
         &Box11,               
         &Box12                
         };

  TScreen                Screen3;
  TLabel                 Label3;
char Label3_Caption[8] = "Screen3";

  TButton               Button5;
char Button5_Caption[8] = "Screen2";

  TButton               Button6;
char Button6_Caption[8] = "Screen4";

  TBox                   Box5;
  TBox                   Box6;
  TBox                   Box7;
  TBox                   Box8;
  TButton                * const code Screen3_Buttons[2]=
         {
         &Button5,             
         &Button6              
         };
  TLabel                 * const code Screen3_Labels[1]=
         {
         &Label3               
         };
  TBox                   * const code Screen3_Boxes[4]=
         {
         &Box5,                
         &Box6,                
         &Box7,                
         &Box8                 
         };

  TScreen                Screen4;
  TLabel                 Label4;
char Label4_Caption[8] = "Screen4";

  TButton               Button7;
char Button7_Caption[8] = "Screen3";

  TButton               Button8;
char Button8_Caption[8] = "Screen1";

  TBox                   Box13;
  TBox                   Box14;
  TBox                   Box15;
  TBox                   Box16;
  TButton                * const code Screen4_Buttons[2]=
         {
         &Button7,             
         &Button8              
         };
  TLabel                 * const code Screen4_Labels[1]=
         {
         &Label4               
         };
  TBox                   * const code Screen4_Boxes[4]=
         {
         &Box13,               
         &Box14,               
         &Box15,               
         &Box16                
         };



static void InitializeObjects() {
  Screen1.Color                     = 0x0013;
  Screen1.Width                     = 800;
  Screen1.Height                    = 480;
  Screen1.ButtonsCount              = 2;
  Screen1.Buttons                   = Screen1_Buttons;
  Screen1.LabelsCount               = 1;
  Screen1.Labels                    = Screen1_Labels;
  Screen1.BoxesCount                = 4;
  Screen1.Boxes                     = Screen1_Boxes;
  Screen1.ObjectsCount              = 7;
  Screen1.OnSwipeUpPtr              = 0;
  Screen1.OnSwipeDownPtr            = 0;
  Screen1.OnSwipeLeftPtr            = 0;
  Screen1.OnSwipeRightPtr           = 0;
  Screen1.OnZoomInPtr               = 0;
  Screen1.OnZoomOutPtr              = 0;
  
  Screen2.Color                     = 0x9806;
  Screen2.Width                     = 480;
  Screen2.Height                    = 800;
  Screen2.ButtonsCount              = 2;
  Screen2.Buttons                   = Screen2_Buttons;
  Screen2.LabelsCount               = 1;
  Screen2.Labels                    = Screen2_Labels;
  Screen2.BoxesCount                = 4;
  Screen2.Boxes                     = Screen2_Boxes;
  Screen2.ObjectsCount              = 7;
  Screen2.OnSwipeUpPtr              = 0;
  Screen2.OnSwipeDownPtr            = 0;
  Screen2.OnSwipeLeftPtr            = 0;
  Screen2.OnSwipeRightPtr           = 0;
  Screen2.OnZoomInPtr               = 0;
  Screen2.OnZoomOutPtr              = 0;
  
  Screen3.Color                     = 0x34D3;
  Screen3.Width                     = 800;
  Screen3.Height                    = 480;
  Screen3.ButtonsCount              = 2;
  Screen3.Buttons                   = Screen3_Buttons;
  Screen3.LabelsCount               = 1;
  Screen3.Labels                    = Screen3_Labels;
  Screen3.BoxesCount                = 4;
  Screen3.Boxes                     = Screen3_Boxes;
  Screen3.ObjectsCount              = 7;
  Screen3.OnSwipeUpPtr              = 0;
  Screen3.OnSwipeDownPtr            = 0;
  Screen3.OnSwipeLeftPtr            = 0;
  Screen3.OnSwipeRightPtr           = 0;
  Screen3.OnZoomInPtr               = 0;
  Screen3.OnZoomOutPtr              = 0;

  Screen4.Color                     = 0x9E7F;
  Screen4.Width                     = 480;
  Screen4.Height                    = 800;
  Screen4.ButtonsCount              = 2;
  Screen4.Buttons                   = Screen4_Buttons;
  Screen4.LabelsCount               = 1;
  Screen4.Labels                    = Screen4_Labels;
  Screen4.BoxesCount                = 4;
  Screen4.Boxes                     = Screen4_Boxes;
  Screen4.ObjectsCount              = 7;
  Screen4.OnSwipeUpPtr              = 0;
  Screen4.OnSwipeDownPtr            = 0;
  Screen4.OnSwipeLeftPtr            = 0;
  Screen4.OnSwipeRightPtr           = 0;
  Screen4.OnZoomInPtr               = 0;
  Screen4.OnZoomOutPtr              = 0;

  Label1.OwnerScreen     = &Screen1;
  Label1.Order           = 0;
  Label1.Left            = 322;
  Label1.Top             = 20;
  Label1.Width           = 107;
  Label1.Height          = 36;
  Label1.Visible         = 1;
  Label1.Active          = 1;
  Label1.Caption         = Label1_Caption;
  Label1.FontName        = Tahoma26x33_Regular;
  Label1.Font_Color      = 0xFFFF;
  Label1.VerticalText    = 0;

  Button1.OwnerScreen     = &Screen1;
  Button1.Order           = 1;
  Button1.Left            = 69;
  Button1.Top             = 197;
  Button1.Width           = 193;
  Button1.Height          = 81;
  Button1.Pen_Width       = 1;
  Button1.Pen_Color       = 0x0000;
  Button1.Visible         = 1;
  Button1.Active          = 1;
  Button1.Transparent     = 1;
  Button1.Caption         = Button1_Caption;
  Button1.TextAlign       = _taCenter;
  Button1.TextAlignVertical= _tavMiddle;
  Button1.FontName        = Tahoma19x23_Regular;
  Button1.PressColEnabled = 1;
  Button1.Font_Color      = 0x0000;
  Button1.VerticalText    = 0;
  Button1.Gradient        = 1;
  Button1.Gradient_Orientation = 0;
  Button1.Gradient_Start_Color = 0xFFFF;
  Button1.Gradient_End_Color = 0xC618;
  Button1.Color           = 0xC618;
  Button1.Press_Color     = 0xE71C;
  Button1.OnClickPtr      = Button1OnClick;

  Button2.OwnerScreen     = &Screen1;
  Button2.Order           = 2;
  Button2.Left            = 526;
  Button2.Top             = 190;
  Button2.Width           = 216;
  Button2.Height          = 87;
  Button2.Pen_Width       = 1;
  Button2.Pen_Color       = 0x0000;
  Button2.Visible         = 1;
  Button2.Active          = 1;
  Button2.Transparent     = 1;
  Button2.Caption         = Button2_Caption;
  Button2.TextAlign       = _taCenter;
  Button2.TextAlignVertical= _tavMiddle;
  Button2.FontName        = Tahoma19x23_Regular;
  Button2.PressColEnabled = 1;
  Button2.Font_Color      = 0x0000;
  Button2.VerticalText    = 0;
  Button2.Gradient        = 1;
  Button2.Gradient_Orientation = 0;
  Button2.Gradient_Start_Color = 0xFFFF;
  Button2.Gradient_End_Color = 0xC618;
  Button2.Color           = 0xC618;
  Button2.Press_Color     = 0xE71C;
  Button2.OnClickPtr      = Button2OnClick;

  Box1.OwnerScreen     = &Screen1;
  Box1.Order           = 3;
  Box1.Left            = 618;
  Box1.Top             = 13;
  Box1.Width           = 171;
  Box1.Height          = 57;
  Box1.Pen_Width       = 1;
  Box1.Pen_Color       = 0x0000;
  Box1.Visible         = 1;
  Box1.Active          = 1;
  Box1.Transparent     = 1;
  Box1.Gradient        = 1;
  Box1.Gradient_Orientation = 0;
  Box1.Gradient_Start_Color = 0xFFFF;
  Box1.Gradient_End_Color = 0xC618;
  Box1.Color           = 0xC618;
  Box1.PressColEnabled = 1;
  Box1.Press_Color     = 0xE71C;

  Box2.OwnerScreen     = &Screen1;
  Box2.Order           = 4;
  Box2.Left            = 17;
  Box2.Top             = 13;
  Box2.Width           = 176;
  Box2.Height          = 61;
  Box2.Pen_Width       = 1;
  Box2.Pen_Color       = 0x0000;
  Box2.Visible         = 1;
  Box2.Active          = 1;
  Box2.Transparent     = 1;
  Box2.Gradient        = 1;
  Box2.Gradient_Orientation = 0;
  Box2.Gradient_Start_Color = 0xFFFF;
  Box2.Gradient_End_Color = 0xC618;
  Box2.Color           = 0xC618;
  Box2.PressColEnabled = 1;
  Box2.Press_Color     = 0xE71C;

  Box3.OwnerScreen     = &Screen1;
  Box3.Order           = 5;
  Box3.Left            = 14;
  Box3.Top             = 387;
  Box3.Width           = 193;
  Box3.Height          = 74;
  Box3.Pen_Width       = 1;
  Box3.Pen_Color       = 0x0000;
  Box3.Visible         = 1;
  Box3.Active          = 1;
  Box3.Transparent     = 1;
  Box3.Gradient        = 1;
  Box3.Gradient_Orientation = 0;
  Box3.Gradient_Start_Color = 0xFFFF;
  Box3.Gradient_End_Color = 0xC618;
  Box3.Color           = 0xC618;
  Box3.PressColEnabled = 1;
  Box3.Press_Color     = 0xE71C;

  Box4.OwnerScreen     = &Screen1;
  Box4.Order           = 6;
  Box4.Left            = 611;
  Box4.Top             = 396;
  Box4.Width           = 166;
  Box4.Height          = 58;
  Box4.Pen_Width       = 1;
  Box4.Pen_Color       = 0x0000;
  Box4.Visible         = 1;
  Box4.Active          = 1;
  Box4.Transparent     = 1;
  Box4.Gradient        = 1;
  Box4.Gradient_Orientation = 0;
  Box4.Gradient_Start_Color = 0xFFFF;
  Box4.Gradient_End_Color = 0xC618;
  Box4.Color           = 0xC618;
  Box4.PressColEnabled = 1;
  Box4.Press_Color     = 0xE71C;

  Label2.OwnerScreen     = &Screen2;
  Label2.Order           = 0;
  Label2.Left            = 167;
  Label2.Top             = 36;
  Label2.Width           = 115;
  Label2.Height          = 39;
  Label2.Visible         = 1;
  Label2.Active          = 1;
  Label2.Caption         = Label2_Caption;
  Label2.FontName        = Tahoma28x35_Regular;
  Label2.Font_Color      = 0xFFFF;
  Label2.VerticalText    = 0;

  Button3.OwnerScreen     = &Screen2;
  Button3.Order           = 1;
  Button3.Left            = 153;
  Button3.Top             = 152;
  Button3.Width           = 178;
  Button3.Height          = 63;
  Button3.Pen_Width       = 1;
  Button3.Pen_Color       = 0x0000;
  Button3.Visible         = 1;
  Button3.Active          = 1;
  Button3.Transparent     = 1;
  Button3.Caption         = Button3_Caption;
  Button3.TextAlign       = _taCenter;
  Button3.TextAlignVertical= _tavMiddle;
  Button3.FontName        = Tahoma19x23_Regular;
  Button3.PressColEnabled = 1;
  Button3.Font_Color      = 0x0000;
  Button3.VerticalText    = 0;
  Button3.Gradient        = 1;
  Button3.Gradient_Orientation = 0;
  Button3.Gradient_Start_Color = 0xFFFF;
  Button3.Gradient_End_Color = 0xC618;
  Button3.Color           = 0xC618;
  Button3.Press_Color     = 0xE71C;
  Button3.OnClickPtr      = Button3OnClick;

  Button4.OwnerScreen     = &Screen2;
  Button4.Order           = 2;
  Button4.Left            = 160;
  Button4.Top             = 568;
  Button4.Width           = 168;
  Button4.Height          = 71;
  Button4.Pen_Width       = 1;
  Button4.Pen_Color       = 0x0000;
  Button4.Visible         = 1;
  Button4.Active          = 1;
  Button4.Transparent     = 1;
  Button4.Caption         = Button4_Caption;
  Button4.TextAlign       = _taCenter;
  Button4.TextAlignVertical= _tavMiddle;
  Button4.FontName        = Tahoma19x23_Regular;
  Button4.PressColEnabled = 1;
  Button4.Font_Color      = 0x0000;
  Button4.VerticalText    = 0;
  Button4.Gradient        = 1;
  Button4.Gradient_Orientation = 0;
  Button4.Gradient_Start_Color = 0xFFFF;
  Button4.Gradient_End_Color = 0xC618;
  Button4.Color           = 0xC618;
  Button4.Press_Color     = 0xE71C;
  Button4.OnClickPtr      = Button4OnClick;

  Box9.OwnerScreen     = &Screen2;
  Box9.Order           = 3;
  Box9.Left            = 9;
  Box9.Top             = 9;
  Box9.Width           = 75;
  Box9.Height          = 175;
  Box9.Pen_Width       = 1;
  Box9.Pen_Color       = 0x0000;
  Box9.Visible         = 1;
  Box9.Active          = 1;
  Box9.Transparent     = 1;
  Box9.Gradient        = 1;
  Box9.Gradient_Orientation = 0;
  Box9.Gradient_Start_Color = 0xFFFF;
  Box9.Gradient_End_Color = 0xC618;
  Box9.Color           = 0xC618;
  Box9.PressColEnabled = 1;
  Box9.Press_Color     = 0xE71C;

  Box10.OwnerScreen     = &Screen2;
  Box10.Order           = 4;
  Box10.Left            = 393;
  Box10.Top             = 10;
  Box10.Width           = 72;
  Box10.Height          = 148;
  Box10.Pen_Width       = 1;
  Box10.Pen_Color       = 0x0000;
  Box10.Visible         = 1;
  Box10.Active          = 1;
  Box10.Transparent     = 1;
  Box10.Gradient        = 1;
  Box10.Gradient_Orientation = 0;
  Box10.Gradient_Start_Color = 0xFFFF;
  Box10.Gradient_End_Color = 0xC618;
  Box10.Color           = 0xC618;
  Box10.PressColEnabled = 1;
  Box10.Press_Color     = 0xE71C;

  Box11.OwnerScreen     = &Screen2;
  Box11.Order           = 5;
  Box11.Left            = 20;
  Box11.Top             = 613;
  Box11.Width           = 78;
  Box11.Height          = 157;
  Box11.Pen_Width       = 1;
  Box11.Pen_Color       = 0x0000;
  Box11.Visible         = 1;
  Box11.Active          = 1;
  Box11.Transparent     = 1;
  Box11.Gradient        = 1;
  Box11.Gradient_Orientation = 0;
  Box11.Gradient_Start_Color = 0xFFFF;
  Box11.Gradient_End_Color = 0xC618;
  Box11.Color           = 0xC618;
  Box11.PressColEnabled = 1;
  Box11.Press_Color     = 0xE71C;

  Box12.OwnerScreen     = &Screen2;
  Box12.Order           = 6;
  Box12.Left            = 385;
  Box12.Top             = 613;
  Box12.Width           = 74;
  Box12.Height          = 164;
  Box12.Pen_Width       = 1;
  Box12.Pen_Color       = 0x0000;
  Box12.Visible         = 1;
  Box12.Active          = 1;
  Box12.Transparent     = 1;
  Box12.Gradient        = 1;
  Box12.Gradient_Orientation = 0;
  Box12.Gradient_Start_Color = 0xFFFF;
  Box12.Gradient_End_Color = 0xC618;
  Box12.Color           = 0xC618;
  Box12.PressColEnabled = 1;
  Box12.Press_Color     = 0xE71C;

  Label3.OwnerScreen     = &Screen3;
  Label3.Order           = 0;
  Label3.Left            = 339;
  Label3.Top             = 30;
  Label3.Width           = 115;
  Label3.Height          = 39;
  Label3.Visible         = 1;
  Label3.Active          = 1;
  Label3.Caption         = Label3_Caption;
  Label3.FontName        = Tahoma28x35_Regular;
  Label3.Font_Color      = 0x0000;
  Label3.VerticalText    = 0;

  Button5.OwnerScreen     = &Screen3;
  Button5.Order           = 1;
  Button5.Left            = 100;
  Button5.Top             = 206;
  Button5.Width           = 177;
  Button5.Height          = 76;
  Button5.Pen_Width       = 1;
  Button5.Pen_Color       = 0x0000;
  Button5.Visible         = 1;
  Button5.Active          = 1;
  Button5.Transparent     = 1;
  Button5.Caption         = Button5_Caption;
  Button5.TextAlign       = _taCenter;
  Button5.TextAlignVertical= _tavMiddle;
  Button5.FontName        = Tahoma19x23_Regular;
  Button5.PressColEnabled = 1;
  Button5.Font_Color      = 0x0000;
  Button5.VerticalText    = 0;
  Button5.Gradient        = 1;
  Button5.Gradient_Orientation = 0;
  Button5.Gradient_Start_Color = 0xFFFF;
  Button5.Gradient_End_Color = 0xC618;
  Button5.Color           = 0xC618;
  Button5.Press_Color     = 0xE71C;
  Button5.OnClickPtr      = Button2OnClick;

  Button6.OwnerScreen     = &Screen3;
  Button6.Order           = 2;
  Button6.Left            = 518;
  Button6.Top             = 205;
  Button6.Width           = 168;
  Button6.Height          = 80;
  Button6.Pen_Width       = 1;
  Button6.Pen_Color       = 0x0000;
  Button6.Visible         = 1;
  Button6.Active          = 1;
  Button6.Transparent     = 1;
  Button6.Caption         = Button6_Caption;
  Button6.TextAlign       = _taCenter;
  Button6.TextAlignVertical= _tavMiddle;
  Button6.FontName        = Tahoma19x23_Regular;
  Button6.PressColEnabled = 1;
  Button6.Font_Color      = 0x0000;
  Button6.VerticalText    = 0;
  Button6.Gradient        = 1;
  Button6.Gradient_Orientation = 0;
  Button6.Gradient_Start_Color = 0xFFFF;
  Button6.Gradient_End_Color = 0xC618;
  Button6.Color           = 0xC618;
  Button6.Press_Color     = 0xE71C;
  Button6.OnClickPtr      = Button1OnClick;

  Box5.OwnerScreen     = &Screen3;
  Box5.Order           = 3;
  Box5.Left            = 19;
  Box5.Top             = 9;
  Box5.Width           = 174;
  Box5.Height          = 64;
  Box5.Pen_Width       = 1;
  Box5.Pen_Color       = 0x0000;
  Box5.Visible         = 1;
  Box5.Active          = 1;
  Box5.Transparent     = 1;
  Box5.Gradient        = 1;
  Box5.Gradient_Orientation = 0;
  Box5.Gradient_Start_Color = 0xFFFF;
  Box5.Gradient_End_Color = 0xC618;
  Box5.Color           = 0xC618;
  Box5.PressColEnabled = 1;
  Box5.Press_Color     = 0xE71C;

  Box6.OwnerScreen     = &Screen3;
  Box6.Order           = 4;
  Box6.Left            = 629;
  Box6.Top             = 10;
  Box6.Width           = 159;
  Box6.Height          = 61;
  Box6.Pen_Width       = 1;
  Box6.Pen_Color       = 0x0000;
  Box6.Visible         = 1;
  Box6.Active          = 1;
  Box6.Transparent     = 1;
  Box6.Gradient        = 1;
  Box6.Gradient_Orientation = 0;
  Box6.Gradient_Start_Color = 0xFFFF;
  Box6.Gradient_End_Color = 0xC618;
  Box6.Color           = 0xC618;
  Box6.PressColEnabled = 1;
  Box6.Press_Color     = 0xE71C;

  Box7.OwnerScreen     = &Screen3;
  Box7.Order           = 5;
  Box7.Left            = 11;
  Box7.Top             = 386;
  Box7.Width           = 189;
  Box7.Height          = 76;
  Box7.Pen_Width       = 1;
  Box7.Pen_Color       = 0x0000;
  Box7.Visible         = 1;
  Box7.Active          = 1;
  Box7.Transparent     = 1;
  Box7.Gradient        = 1;
  Box7.Gradient_Orientation = 0;
  Box7.Gradient_Start_Color = 0xFFFF;
  Box7.Gradient_End_Color = 0xC618;
  Box7.Color           = 0xC618;
  Box7.PressColEnabled = 1;
  Box7.Press_Color     = 0xE71C;

  Box8.OwnerScreen     = &Screen3;
  Box8.Order           = 6;
  Box8.Left            = 616;
  Box8.Top             = 388;
  Box8.Width           = 166;
  Box8.Height          = 75;
  Box8.Pen_Width       = 1;
  Box8.Pen_Color       = 0x0000;
  Box8.Visible         = 1;
  Box8.Active          = 1;
  Box8.Transparent     = 1;
  Box8.Gradient        = 1;
  Box8.Gradient_Orientation = 0;
  Box8.Gradient_Start_Color = 0xFFFF;
  Box8.Gradient_End_Color = 0xC618;
  Box8.Color           = 0xC618;
  Box8.PressColEnabled = 1;
  Box8.Press_Color     = 0xE71C;

  Label4.OwnerScreen     = &Screen4;
  Label4.Order           = 0;
  Label4.Left            = 175;
  Label4.Top             = 35;
  Label4.Width           = 115;
  Label4.Height          = 39;
  Label4.Visible         = 1;
  Label4.Active          = 1;
  Label4.Caption         = Label4_Caption;
  Label4.FontName        = Tahoma28x35_Regular;
  Label4.Font_Color      = 0x0000;
  Label4.VerticalText    = 0;

  Button7.OwnerScreen     = &Screen4;
  Button7.Order           = 1;
  Button7.Left            = 157;
  Button7.Top             = 179;
  Button7.Width           = 160;
  Button7.Height          = 74;
  Button7.Pen_Width       = 1;
  Button7.Pen_Color       = 0x0000;
  Button7.Visible         = 1;
  Button7.Active          = 1;
  Button7.Transparent     = 1;
  Button7.Caption         = Button7_Caption;
  Button7.TextAlign       = _taCenter;
  Button7.TextAlignVertical= _tavMiddle;
  Button7.FontName        = Tahoma19x23_Regular;
  Button7.PressColEnabled = 1;
  Button7.Font_Color      = 0x0000;
  Button7.VerticalText    = 0;
  Button7.Gradient        = 1;
  Button7.Gradient_Orientation = 0;
  Button7.Gradient_Start_Color = 0xFFFF;
  Button7.Gradient_End_Color = 0xC618;
  Button7.Color           = 0xC618;
  Button7.Press_Color     = 0xE71C;
  Button7.OnClickPtr      = Button4OnClick;

  Button8.OwnerScreen     = &Screen4;
  Button8.Order           = 2;
  Button8.Left            = 159;
  Button8.Top             = 565;
  Button8.Width           = 150;
  Button8.Height          = 70;
  Button8.Pen_Width       = 1;
  Button8.Pen_Color       = 0x0000;
  Button8.Visible         = 1;
  Button8.Active          = 1;
  Button8.Transparent     = 1;
  Button8.Caption         = Button8_Caption;
  Button8.TextAlign       = _taCenter;
  Button8.TextAlignVertical= _tavMiddle;
  Button8.FontName        = Tahoma19x23_Regular;
  Button8.PressColEnabled = 1;
  Button8.Font_Color      = 0x0000;
  Button8.VerticalText    = 0;
  Button8.Gradient        = 1;
  Button8.Gradient_Orientation = 0;
  Button8.Gradient_Start_Color = 0xFFFF;
  Button8.Gradient_End_Color = 0xC618;
  Button8.Color           = 0xC618;
  Button8.Press_Color     = 0xE71C;
  Button8.OnClickPtr      = Button3OnClick;

  Box13.OwnerScreen     = &Screen4;
  Box13.Order           = 3;
  Box13.Left            = 9;
  Box13.Top             = 13;
  Box13.Width           = 66;
  Box13.Height          = 164;
  Box13.Pen_Width       = 1;
  Box13.Pen_Color       = 0x0000;
  Box13.Visible         = 1;
  Box13.Active          = 1;
  Box13.Transparent     = 1;
  Box13.Gradient        = 1;
  Box13.Gradient_Orientation = 0;
  Box13.Gradient_Start_Color = 0xFFFF;
  Box13.Gradient_End_Color = 0xC618;
  Box13.Color           = 0xC618;
  Box13.PressColEnabled = 1;
  Box13.Press_Color     = 0xE71C;

  Box14.OwnerScreen     = &Screen4;
  Box14.Order           = 4;
  Box14.Left            = 392;
  Box14.Top             = 13;
  Box14.Width           = 67;
  Box14.Height          = 157;
  Box14.Pen_Width       = 1;
  Box14.Pen_Color       = 0x0000;
  Box14.Visible         = 1;
  Box14.Active          = 1;
  Box14.Transparent     = 1;
  Box14.Gradient        = 1;
  Box14.Gradient_Orientation = 0;
  Box14.Gradient_Start_Color = 0xFFFF;
  Box14.Gradient_End_Color = 0xC618;
  Box14.Color           = 0xC618;
  Box14.PressColEnabled = 1;
  Box14.Press_Color     = 0xE71C;

  Box15.OwnerScreen     = &Screen4;
  Box15.Order           = 5;
  Box15.Left            = 11;
  Box15.Top             = 617;
  Box15.Width           = 71;
  Box15.Height          = 165;
  Box15.Pen_Width       = 1;
  Box15.Pen_Color       = 0x0000;
  Box15.Visible         = 1;
  Box15.Active          = 1;
  Box15.Transparent     = 1;
  Box15.Gradient        = 1;
  Box15.Gradient_Orientation = 0;
  Box15.Gradient_Start_Color = 0xFFFF;
  Box15.Gradient_End_Color = 0xC618;
  Box15.Color           = 0xC618;
  Box15.PressColEnabled = 1;
  Box15.Press_Color     = 0xE71C;

  Box16.OwnerScreen     = &Screen4;
  Box16.Order           = 6;
  Box16.Left            = 390;
  Box16.Top             = 613;
  Box16.Width           = 71;
  Box16.Height          = 169;
  Box16.Pen_Width       = 1;
  Box16.Pen_Color       = 0x0000;
  Box16.Visible         = 1;
  Box16.Active          = 1;
  Box16.Transparent     = 1;
  Box16.Gradient        = 1;
  Box16.Gradient_Orientation = 0;
  Box16.Gradient_Start_Color = 0xFFFF;
  Box16.Gradient_End_Color = 0xC618;
  Box16.Color           = 0xC618;
  Box16.PressColEnabled = 1;
  Box16.Press_Color     = 0xE71C;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetButton(index)              CurrentScreen->Buttons[index]
#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]

void DrawButton(TButton *Abutton) {
unsigned int ALeft, ATop;
  if (Abutton->Visible != 0) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(Abutton->Transparent, Abutton->Press_Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_End_Color, Abutton->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(Abutton->Transparent, Abutton->Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_Start_Color, Abutton->Gradient_End_Color);
    }
    TFT_Set_Pen(Abutton->Pen_Color, Abutton->Pen_Width);
    TFT_Rectangle(Abutton->Left, Abutton->Top, Abutton->Left + Abutton->Width - 1, Abutton->Top + Abutton->Height - 1);
    if (Abutton->VerticalText != 0)
      TFT_Set_Font(Abutton->FontName, Abutton->Font_Color, FO_VERTICAL_COLUMN);
    else
      TFT_Set_Font(Abutton->FontName, Abutton->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(Abutton->Caption, Abutton->Left, Abutton->Top);
    if (Abutton->TextAlign == _taLeft)
      ALeft = Abutton->Left + 4;
    else if (Abutton->TextAlign == _taCenter)
      ALeft = Abutton->Left + (Abutton->Width - caption_length) / 2;
    else if (Abutton->TextAlign == _taRight)
      ALeft = Abutton->Left + Abutton->Width - caption_length - 4;

    if (Abutton->TextAlignVertical == _tavTop)
      ATop = Abutton->Top + 4;
    else if (Abutton->TextAlignVertical == _tavMiddle)
      ATop = Abutton->Top + ((Abutton->Height - caption_height) / 2);
    else if (Abutton->TextAlignVertical == _tavBottom)
      ATop = Abutton->Top + (Abutton->Height - caption_height - 4);

    TFT_Write_Text(Abutton->Caption, ALeft, ATop);
  }
}

void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible != 0) {
    if (ALabel->VerticalText != 0)
      TFT_Set_Font(ALabel->FontName, ALabel->Font_Color, FO_VERTICAL_COLUMN);
    else
      TFT_Set_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void DrawBox(TBox *ABox) {
  if (ABox->Visible != 0) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(ABox->Transparent, ABox->Press_Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_End_Color, ABox->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    }
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short button_idx;
  TButton *local_button;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short box_idx;
  TBox *local_box;
  char save_bled;

  object_pressed = 0;
  order = 0;
  button_idx = 0;
  label_idx = 0;
  box_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    TFT_BLED           = 0;
    TFT_Set_Active(Set_Index, Write_Command, Write_Data);
    TFT_Init_SSD1963_Board_50(CurrentScreen->Width, CurrentScreen->Height);
    FT5XX6_SetSize(CurrentScreen->Width, CurrentScreen->Height);
    TFT_Fill_Screen(CurrentScreen->Color);
    TFT_Set_DBC_SSD1963(255);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (button_idx < CurrentScreen->ButtonsCount) {
      local_button = GetButton(button_idx);
      if (order == local_button->Order) {
        button_idx++;
        order++;
        DrawButton(local_button);
      }
    }

    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  button_order        = -1;
  label_order         = -1;
  box_order           = -1;
  //  Buttons
  for ( _object_count = 0 ; _object_count < CurrentScreen->ButtonsCount ; _object_count++ ) {
    local_button = GetButton(_object_count);
    if (local_button->Active != 0) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  //  Labels
  for ( _object_count = 0 ; _object_count < CurrentScreen->LabelsCount ; _object_count++ ) {
    local_label = GetLabel(_object_count);
    if (local_label->Active != 0) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  //  Boxes
  for ( _object_count = 0 ; _object_count < CurrentScreen->BoxesCount ; _object_count++ ) {
    local_box = GetBox(_object_count);
    if (local_box->Active != 0) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  _object_count = -1;
  if (button_order >  _object_count )
    _object_count = button_order;
  if (label_order >  _object_count )
    _object_count = label_order;
  if (box_order >  _object_count )
    _object_count = box_order;
}


void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_button         = 0;
  exec_label          = 0;
  exec_box            = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == button_order) {
      if (exec_button->Active != 0) {
        if (exec_button->OnPressPtr != 0) {
          exec_button->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active != 0) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active != 0) {
        if (exec_box->OnPressPtr != 0) {
          exec_box->OnPressPtr();
          return;
        }
      }
    }

  }
}

void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Button
    case 0: {
      if (PressedObject != 0) {
        exec_button = (TButton*)PressedObject;
        if ((exec_button->PressColEnabled != 0) && (exec_button->OwnerScreen == CurrentScreen)) {
          DrawButton(exec_button);
        }
        break;
      }
      break;
    }
    // Box
    case 6: {
      if (PressedObject != 0) {
        exec_box = (TBox*)PressedObject;
        if ((exec_box->PressColEnabled != 0) && (exec_box->OwnerScreen == CurrentScreen)) {
          DrawBox(exec_box);
        }
        break;
      }
      break;
    }
  }

  exec_label          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // Buttons
    if (_object_count == button_order) {
      if (exec_button->Active != 0) {
        if (exec_button->OnUpPtr != 0)
          exec_button->OnUpPtr();
        if (PressedObject == (void *)exec_button)
          if (exec_button->OnClickPtr != 0)
            exec_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Labels
    if (_object_count == label_order) {
      if (exec_label->Active != 0) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (void *)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes
    if (_object_count == box_order) {
      if (exec_box->Active != 0) {
        if (exec_box->OnUpPtr != 0)
          exec_box->OnUpPtr();
        if (PressedObject == (void *)exec_box)
          if (exec_box->OnClickPtr != 0)
            exec_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_button         = 0;
  exec_label          = 0;
  exec_box            = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == button_order) {
      if (exec_button->Active != 0) {
        if (exec_button->PressColEnabled != 0) {
          object_pressed = 1;
          DrawButton(exec_button);
        }
        PressedObject = (void *)exec_button;
        PressedObjectType = 0;
        if (exec_button->OnDownPtr != 0) {
          exec_button->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == label_order) {
      if (exec_label->Active != 0) {
        PressedObject = (void *)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active != 0) {
        if (exec_box->PressColEnabled != 0) {
          object_pressed = 1;
          DrawBox(exec_box);
        }
        PressedObject = (void *)exec_box;
        PressedObjectType = 6;
        if (exec_box->OnDownPtr != 0) {
          exec_box->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Process_TP_Gesture(unsigned short aGestID) {
  if ((aGestID == ft5xx6_gest_move_up) && (CurrentScreen->OnSwipeUpPtr != 0))
     CurrentScreen->OnSwipeUpPtr();
  else if ((aGestID == ft5xx6_gest_move_left) && (CurrentScreen->OnSwipeLeftPtr != 0))
       CurrentScreen->OnSwipeLeftPtr();
  else if ((aGestID == ft5xx6_gest_move_down) && (CurrentScreen->OnSwipeDownPtr != 0))
       CurrentScreen->OnSwipeDownPtr();
  else if ((aGestID == ft5xx6_gest_move_right) && (CurrentScreen->OnSwipeRightPtr != 0))
       CurrentScreen->OnSwipeRightPtr();
  else if ((aGestID == FT5XX6_GEST_ZOOM_IN) && (CurrentScreen->OnZoomInPtr != 0))
       CurrentScreen->OnZoomInPtr();
  else if ((aGestID == FT5XX6_GEST_ZOOM_OUT) && (CurrentScreen->OnZoomOutPtr != 0))
       CurrentScreen->OnZoomOutPtr();
}

void Check_TP() {
unsigned short gestureid;
  if (FT5XX6_PressDetect()) {
    gestureid = FT5XX6_GetGesture();
    if (gestureid) {
      Process_TP_Gesture(gestureid);
    }
    if (FT5XX6_GetCoordinates(&Xcoord, &Ycoord) == 0) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU()
{
  I2C0_Init_Advanced(400000, &_GPIO_MODULE_I2C0_PB2_3);
}

char FT5XX6_Config()  {
  FT5XX6_SetI2CAddress(0x38);
  if (FT5XX6_IsOperational() != FT5XX6_OK)
    return  FT5XX6_FAILURE;
  FT5XX6_SetDefaultMode();
  FT5XX6_SetController(_TC_FT5X06);
  FT5XX6_SetSizeAndRotation(800,480,2);
  return FT5XX6_OK;
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();
  if (FT5XX6_Config() == FT5XX6_OK) {
  } else {
    while(1);
  }


  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
  
  TFT_Set_Font(&Tahoma28x35_Regular, CL_YELLOW, FO_HORIZONTAL);
  TFT_Write_Text("HELLO WORLD !", 320, 80);
}